@@ -22,21 +22,45 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
+#ifdef __sgi
+#define errx(exitcode, format, args...)                                        \
+  {                                                                            \
+    warnx(format, ##args);                                                     \
+    exit(exitcode);                                                            \
+  }
+#define warn(format, args...) warnx(format ": %s", ##args, strerror(errno))
+#define warnx(format, args...) fprintf(stderr, format "\n", ##args)
+#endif
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif /* !HAVE_CONFIG_H */
+#endif /* HAVE_CONFIG_H */
 
 #include <sys/types.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif /* HAVE_SYS_SOCKET_H */
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif /* HAVE_NETDB_H */
 #include <signal.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif /* HAVE_UNISTD_H */
 #include <sys/stat.h>
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
 #include <ctype.h>
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif /* HAVE_NETINET_IN_H */
 #include <netinet/tcp.h>
+#ifndef __sgi
 #include <err.h>
+#endif
+#include <string.h>
+#include <errno.h>
 
 #include <openssl/ssl.h>
 #include <openssl/err.h>
@@ -198,7 +222,8 @@ static http2_session_data *create_http2_session_data(app_context *app_ctx,
   session_data->bev = bufferevent_openssl_socket_new(
       app_ctx->evbase, fd, ssl, BUFFEREVENT_SSL_ACCEPTING,
       BEV_OPT_CLOSE_ON_FREE | BEV_OPT_DEFER_CALLBACKS);
-  rv = getnameinfo(addr, addrlen, host, sizeof(host), NULL, 0, NI_NUMERICHOST);
+  rv = getnameinfo(addr, (socklen_t)addrlen, host, sizeof(host), NULL, 0,
+                   NI_NUMERICHOST);
   if (rv != 0) {
     session_data->client_addr = strdup("(unknown)");
   } else {
@@ -253,7 +278,7 @@ static int session_recv(http2_session_data *session_data) {
     warnx("Fatal error: %s", nghttp2_strerror((int)readlen));
     return -1;
   }
-  if (evbuffer_drain(input, readlen) != 0) {
+  if (evbuffer_drain(input, (size_t)readlen) != 0) {
     warnx("Fatal error: evbuffer_drain failed");
     return -1;
   }
@@ -273,7 +298,7 @@ static ssize_t send_callback(nghttp2_session *session _U_, const uint8_t *data,
     return NGHTTP2_ERR_WOULDBLOCK;
   }
   bufferevent_write(bev, data, length);
-  return length;
+  return (ssize_t)length;
 }
 
 /* Returns nonzero if the string |s| ends with the substring |sub| */
@@ -289,13 +314,13 @@ static int ends_with(const char *s, const char *sub) {
 /* Returns int value of hex string character |c| */
 static uint8_t hex_to_uint(uint8_t c) {
   if ('0' <= c && c <= '9') {
-    return c - '0';
+    return (uint8_t)(c - '0');
   }
   if ('A' <= c && c <= 'F') {
-    return c - 'A' + 10;
+    return (uint8_t)(c - 'A' + 10);
   }
   if ('a' <= c && c <= 'f') {
-    return c - 'a' + 10;
+    return (uint8_t)(c - 'a' + 10);
   }
   return 0;
 }
@@ -313,10 +338,11 @@ static char *percent_decode(const uint8_t *value, size_t valuelen) {
     for (i = 0, j = 0; i < valuelen - 2;) {
       if (value[i] != '%' || !isxdigit(value[i + 1]) ||
           !isxdigit(value[i + 2])) {
-        res[j++] = value[i++];
+        res[j++] = (char)value[i++];
         continue;
       }
-      res[j++] = (hex_to_uint(value[i + 1]) << 4) + hex_to_uint(value[i + 2]);
+      res[j++] =
+          (char)((hex_to_uint(value[i + 1]) << 4) + hex_to_uint(value[i + 2]));
       i += 3;
     }
     memcpy(&res[j], &value[i], 2);
@@ -361,8 +387,8 @@ static int send_response(nghttp2_session *session, int32_t stream_id,
   return 0;
 }
 
-const char ERROR_HTML[] = "<html><head><title>404</title></head>"
-                          "<body><h1>404 Not Found</h1></body></html>";
+static const char ERROR_HTML[] = "<html><head><title>404</title></head>"
+                                 "<body><h1>404 Not Found</h1></body></html>";
 
 static int error_reply(nghttp2_session *session,
                        http2_stream_data *stream_data) {
@@ -537,15 +563,8 @@ static int on_stream_close_callback(nghttp2_session *session, int32_t stream_id,
 }
 
 static void initialize_nghttp2_session(http2_session_data *session_data) {
-  nghttp2_option *option;
   nghttp2_session_callbacks *callbacks;
 
-  nghttp2_option_new(&option);
-
-  /* Tells nghttp2_session object that it handles client connection
-     preface */
-  nghttp2_option_set_recv_client_preface(option, 1);
-
   nghttp2_session_callbacks_new(&callbacks);
 
   nghttp2_session_callbacks_set_send_callback(callbacks, send_callback);
@@ -562,11 +581,9 @@ static void initialize_nghttp2_session(http2_session_data *session_data) {
   nghttp2_session_callbacks_set_on_begin_headers_callback(
       callbacks, on_begin_headers_callback);
 
-  nghttp2_session_server_new2(&session_data->session, callbacks, session_data,
-                              option);
+  nghttp2_session_server_new(&session_data->session, callbacks, session_data);
 
   nghttp2_session_callbacks_del(callbacks);
-  nghttp2_option_del(option);
 }
 
 /* Send HTTP/2 client connection header, which includes 24 bytes
@@ -671,13 +688,13 @@ static void start_listen(struct event_base *evbase, const char *service,
 
   rv = getaddrinfo(NULL, service, &hints, &res);
   if (rv != 0) {
-    errx(1, NULL);
+    errx(1, "Could not resolve server address");
   }
   for (rp = res; rp; rp = rp->ai_next) {
     struct evconnlistener *listener;
     listener = evconnlistener_new_bind(
         evbase, acceptcb, app_ctx, LEV_OPT_CLOSE_ON_FREE | LEV_OPT_REUSEABLE,
-        16, rp->ai_addr, rp->ai_addrlen);
+        16, rp->ai_addr, (int)rp->ai_addrlen);
     if (listener) {
       freeaddrinfo(res);
 
@@ -723,8 +740,9 @@ int main(int argc, char **argv) {
   act.sa_handler = SIG_IGN;
   sigaction(SIGPIPE, &act, NULL);
 
+#ifndef OPENSSL_IS_BORINGSSL
   OPENSSL_config(NULL);
-  OpenSSL_add_all_algorithms();
+#endif /* OPENSSL_IS_BORINGSSL */
   SSL_load_error_strings();
   SSL_library_init();
 
