@@ -30,18 +30,30 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif /* !HAVE_CONFIG_H */
+#endif /* HAVE_CONFIG_H */
 
 #include <sys/types.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif /* HAVE_SYS_SOCKET_H */
 #include <sys/stat.h>
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif /* HAVE_NETDB_H */
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif /* HAVE_NETINET_IN_H */
 #include <netinet/tcp.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif /* HAVE_UNISTD_H */
 #include <stdlib.h>
+#ifdef HAVE_TIME_H
 #include <time.h>
+#endif /* HAVE_TIME_H */
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -69,9 +81,9 @@
 #define array_size(a) (sizeof((a)) / sizeof((a)[0]))
 
 /* Returns the length of remaning data in buffer */
-#define io_buf_len(iobuf) ((iobuf)->last - (iobuf)->pos)
+#define io_buf_len(iobuf) ((size_t)((iobuf)->last - (iobuf)->pos))
 /* Returns the space buffer can still accept */
-#define io_buf_left(iobuf) ((iobuf)->end - (iobuf)->last)
+#define io_buf_left(iobuf) ((size_t)((iobuf)->end - (iobuf)->last))
 
 typedef struct {
   /* beginning of buffer */
@@ -148,12 +160,11 @@ typedef struct {
 } timer;
 
 /* document root */
-const char *docroot;
+static const char *docroot;
 /* length of docroot */
-size_t docrootlen;
+static size_t docrootlen;
 
-nghttp2_session_callbacks *shared_callbacks;
-nghttp2_option *shared_option;
+static nghttp2_session_callbacks *shared_callbacks;
 
 static int handle_accept(io_loop *loop, uint32_t events, void *ptr);
 static int handle_connection(io_loop *loop, uint32_t events, void *ptr);
@@ -179,13 +190,57 @@ static char *io_buf_add_str(io_buf *buf, const void *src, size_t len) {
   return (char *)start;
 }
 
-static int memseq(const uint8_t *a, size_t alen, const char *b) {
-  const uint8_t *last = a + alen;
+static int memeq(const void *a, const void *b, size_t n) {
+  return memcmp(a, b, n) == 0;
+}
 
-  for (; a != last && *b && *a == *b; ++a, ++b)
-    ;
+#define streq(A, B, N) ((sizeof((A)) - 1) == (N) && memeq((A), (B), (N)))
 
-  return a == last && *b == 0;
+typedef enum {
+  NGHTTP2_TOKEN__AUTHORITY,
+  NGHTTP2_TOKEN__METHOD,
+  NGHTTP2_TOKEN__PATH,
+  NGHTTP2_TOKEN__SCHEME,
+  NGHTTP2_TOKEN_HOST
+} nghttp2_token;
+
+/* Inspired by h2o header lookup.  https://github.com/h2o/h2o */
+static int lookup_token(const uint8_t *name, size_t namelen) {
+  switch (namelen) {
+  case 5:
+    switch (name[namelen - 1]) {
+    case 'h':
+      if (streq(":pat", name, 4)) {
+        return NGHTTP2_TOKEN__PATH;
+      }
+      break;
+    }
+    break;
+  case 7:
+    switch (name[namelen - 1]) {
+    case 'd':
+      if (streq(":metho", name, 6)) {
+        return NGHTTP2_TOKEN__METHOD;
+      }
+      break;
+    case 'e':
+      if (streq(":schem", name, 6)) {
+        return NGHTTP2_TOKEN__SCHEME;
+      }
+      break;
+    }
+    break;
+  case 10:
+    switch (name[namelen - 1]) {
+    case 'y':
+      if (streq(":authorit", name, 9)) {
+        return NGHTTP2_TOKEN__AUTHORITY;
+      }
+      break;
+    }
+    break;
+  }
+  return -1;
 }
 
 static char *cpydig(char *buf, int n, size_t len) {
@@ -193,7 +248,7 @@ static char *cpydig(char *buf, int n, size_t len) {
 
   p = buf + len - 1;
   do {
-    *p-- = (n % 10) + '0';
+    *p-- = (char)((n % 10) + '0');
     n /= 10;
   } while (p >= buf);
 
@@ -234,11 +289,11 @@ static size_t http_date(char *buf, time_t t) {
   memcpy(p, " GMT", 4);
   p += 4;
 
-  return p - buf;
+  return (size_t)(p - buf);
 }
 
 static char date[29];
-static char datelen;
+static size_t datelen;
 
 static void update_date() { datelen = http_date(date, time(NULL)); }
 
@@ -264,7 +319,7 @@ static size_t utos(char *buf, size_t len, uint64_t n) {
 
   buf += nwrite - 1;
   do {
-    *buf-- = (n % 10) + '0';
+    *buf-- = (char)((n % 10) + '0');
     n /= 10;
   } while (n);
 
@@ -344,8 +399,7 @@ static connection *connection_new(int fd) {
 
   conn = malloc(sizeof(connection));
 
-  rv = nghttp2_session_server_new2(&conn->session, shared_callbacks, conn,
-                                   shared_option);
+  rv = nghttp2_session_server_new(&conn->session, shared_callbacks, conn);
 
   if (rv != 0) {
     goto cleanup;
@@ -614,19 +668,52 @@ static void stream_error(connection *conn, int32_t stream_id,
                             error_code);
 }
 
+static int send_data_callback(nghttp2_session *session _U_,
+                              nghttp2_frame *frame, const uint8_t *framehd,
+                              size_t length, nghttp2_data_source *source,
+                              void *user_data) {
+  connection *conn = user_data;
+  uint8_t *p = conn->buf.last;
+  stream *strm = source->ptr;
+
+  /* We never use padding in this program */
+  assert(frame->data.padlen == 0);
+
+  if ((size_t)io_buf_left(&conn->buf) < 9 + frame->hd.length) {
+    return NGHTTP2_ERR_WOULDBLOCK;
+  }
+
+  memcpy(p, framehd, 9);
+  p += 9;
+
+  while (length) {
+    ssize_t nread;
+    while ((nread = read(strm->filefd, p, length)) == -1 && errno == EINTR)
+      ;
+    if (nread == -1) {
+      return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
+    }
+
+    length -= (size_t)nread;
+    p += nread;
+  }
+
+  conn->buf.last = p;
+
+  return 0;
+}
+
 static ssize_t fd_read_callback(nghttp2_session *session _U_,
-                                int32_t stream_id _U_, uint8_t *buf,
+                                int32_t stream_id _U_, uint8_t *buf _U_,
                                 size_t length, uint32_t *data_flags,
                                 nghttp2_data_source *source,
                                 void *user_data _U_) {
   stream *strm = source->ptr;
-  ssize_t nread;
+  ssize_t nread =
+      (int64_t)length < strm->fileleft ? (int64_t)length : strm->fileleft;
+
+  *data_flags |= NGHTTP2_DATA_FLAG_NO_COPY;
 
-  while ((nread = read(strm->filefd, buf, length)) == -1 && errno == EINTR)
-    ;
-  if (nread == -1) {
-    return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-  }
   strm->fileleft -= nread;
   if (nread == 0 || strm->fileleft == 0) {
     if (strm->fileleft != 0) {
@@ -643,7 +730,7 @@ static ssize_t resbuf_read_callback(nghttp2_session *session _U_,
                                     nghttp2_data_source *source,
                                     void *user_data _U_) {
   stream *strm = source->ptr;
-  size_t left = strm->res_end - strm->res_begin;
+  size_t left = (size_t)(strm->res_end - strm->res_begin);
   size_t nwrite = length < left ? length : left;
 
   memcpy(buf, strm->res_begin, nwrite);
@@ -653,7 +740,7 @@ static ssize_t resbuf_read_callback(nghttp2_session *session _U_,
     *data_flags |= NGHTTP2_DATA_FLAG_EOF;
   }
 
-  return nwrite;
+  return (ssize_t)nwrite;
 }
 
 static int hex_digit(char c) {
@@ -663,14 +750,14 @@ static int hex_digit(char c) {
 
 static unsigned int hex_to_uint(char c) {
   if (c <= '9') {
-    return c - '0';
+    return (unsigned int)(c - '0');
   }
 
   if (c <= 'F') {
-    return c - 'A' + 10;
+    return (unsigned int)(c - 'A' + 10);
   }
 
-  return c - 'a' + 10;
+  return (unsigned int)(c - 'a' + 10);
 }
 
 static void percent_decode(io_buf *buf, const char *s) {
@@ -680,12 +767,13 @@ static void percent_decode(io_buf *buf, const char *s) {
     }
 
     if (*s == '%' && hex_digit(*(s + 1)) && hex_digit(*(s + 2))) {
-      *buf->last++ = (hex_to_uint(*(s + 1)) << 4) + hex_to_uint(*(s + 2));
+      *buf->last++ =
+          (uint8_t)((hex_to_uint(*(s + 1)) << 4) + hex_to_uint(*(s + 2)));
       s += 2;
       continue;
     }
 
-    *buf->last++ = *s;
+    *buf->last++ = (uint8_t)*s;
   }
 }
 
@@ -720,7 +808,7 @@ static int make_path(io_buf *pathbuf, const char *req, size_t reqlen _U_) {
 
   *pathbuf->last++ = '\0';
 
-  if (!check_path((const char *)p, pathbuf->last - 1 - p)) {
+  if (!check_path((const char *)p, (size_t)(pathbuf->last - 1 - p))) {
 
     return -1;
   }
@@ -871,7 +959,8 @@ static int process_request(stream *strm, connection *conn) {
   strm->fileleft = stbuf.st_size;
 
   lastmodlen = http_date(lastmod, stbuf.st_mtim.tv_sec);
-  contentlengthlen = utos(contentlength, sizeof(contentlength), stbuf.st_size);
+  contentlengthlen =
+      utos(contentlength, sizeof(contentlength), (uint64_t)stbuf.st_size);
 
   nva[nvlen].value = (uint8_t *)contentlength;
   nva[nvlen].valuelen = contentlengthlen;
@@ -925,8 +1014,7 @@ static int on_header_callback(nghttp2_session *session,
                               const nghttp2_frame *frame, const uint8_t *name,
                               size_t namelen, const uint8_t *value,
                               size_t valuelen, uint8_t flags _U_,
-                              void *user_data) {
-  connection *conn = user_data;
+                              void *user_data _U_) {
   stream *strm;
 
   if (frame->hd.type != NGHTTP2_HEADERS ||
@@ -940,74 +1028,42 @@ static int on_header_callback(nghttp2_session *session,
     return 0;
   }
 
-  if (!nghttp2_check_header_name(name, namelen) ||
-      !nghttp2_check_header_value(value, valuelen)) {
-    return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-  }
-
-  if (memseq(name, namelen, ":method")) {
-    if (strm->method) {
-      stream_error(conn, strm->stream_id, NGHTTP2_PROTOCOL_ERROR);
-      return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-    }
+  switch (lookup_token(name, namelen)) {
+  case NGHTTP2_TOKEN__METHOD:
     strm->method = io_buf_add_str(&strm->scrbuf, value, valuelen);
     if (!strm->method) {
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
     }
     strm->methodlen = valuelen;
-    return 0;
-  }
-
-  if (memseq(name, namelen, ":scheme")) {
-    if (strm->scheme) {
-      stream_error(conn, strm->stream_id, NGHTTP2_PROTOCOL_ERROR);
-      return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-    }
+    break;
+  case NGHTTP2_TOKEN__SCHEME:
     strm->scheme = io_buf_add_str(&strm->scrbuf, value, valuelen);
     if (!strm->scheme) {
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
     }
     strm->schemelen = valuelen;
-    return 0;
-  }
-
-  if (memseq(name, namelen, ":authority")) {
-    if (strm->authority) {
-      stream_error(conn, strm->stream_id, NGHTTP2_PROTOCOL_ERROR);
-      return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-    }
+    break;
+  case NGHTTP2_TOKEN__AUTHORITY:
     strm->authority = io_buf_add_str(&strm->scrbuf, value, valuelen);
     if (!strm->authority) {
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
     }
     strm->authoritylen = valuelen;
-    return 0;
-  }
-
-  if (memseq(name, namelen, ":path")) {
-    if (strm->path) {
-      stream_error(conn, strm->stream_id, NGHTTP2_PROTOCOL_ERROR);
-      return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-    }
+    break;
+  case NGHTTP2_TOKEN__PATH:
     strm->path = io_buf_add_str(&strm->scrbuf, value, valuelen);
     if (!strm->path) {
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
     }
     strm->pathlen = valuelen;
-    return 0;
-  }
-
-  if (name[0] == ':') {
-    stream_error(conn, strm->stream_id, NGHTTP2_PROTOCOL_ERROR);
-    return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-  }
-
-  if (memseq(name, namelen, "host") && !strm->host) {
+    break;
+  case NGHTTP2_TOKEN_HOST:
     strm->host = io_buf_add_str(&strm->scrbuf, value, valuelen);
     if (!strm->host) {
       return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
     }
     strm->hostlen = valuelen;
+    break;
   }
 
   return 0;
@@ -1029,12 +1085,6 @@ static int on_frame_recv_callback(nghttp2_session *session,
     return 0;
   }
 
-  if (!strm->method || !strm->scheme || !strm->path ||
-      (!strm->authority && !strm->host)) {
-    stream_error(conn, strm->stream_id, NGHTTP2_PROTOCOL_ERROR);
-    return 0;
-  }
-
   if (!strm->host) {
     strm->host = strm->authority;
     strm->hostlen = strm->authoritylen;
@@ -1101,7 +1151,7 @@ static int do_read(connection *conn) {
       return -1;
     }
 
-    nproc = nghttp2_session_mem_recv(conn->session, buf, nread);
+    nproc = nghttp2_session_mem_recv(conn->session, buf, (size_t)nread);
 
     if (nproc < 0) {
       return -1;
@@ -1156,13 +1206,13 @@ static int do_write(connection *conn) {
         break;
       }
 
-      if (io_buf_left(&conn->buf) < n) {
+      if (io_buf_left(&conn->buf) < (size_t)n) {
         conn->cache = b;
-        conn->cachelen = n;
+        conn->cachelen = (size_t)n;
         break;
       }
 
-      io_buf_add(&conn->buf, b, n);
+      io_buf_add(&conn->buf, b, (size_t)n);
     }
   }
 }
@@ -1269,14 +1319,8 @@ int main(int argc, char **argv) {
       shared_callbacks, on_stream_close_callback);
   nghttp2_session_callbacks_set_on_frame_not_send_callback(
       shared_callbacks, on_frame_not_send_callback);
-
-  rv = nghttp2_option_new(&shared_option);
-  if (rv != 0) {
-    fprintf(stderr, "nghttp2_option_new: %s", nghttp2_strerror(rv));
-    exit(EXIT_FAILURE);
-  }
-
-  nghttp2_option_set_recv_client_preface(shared_option, 1);
+  nghttp2_session_callbacks_set_send_data_callback(shared_callbacks,
+                                                   send_data_callback);
 
   rv = io_loop_add(&loop, serv.fd, EPOLLIN, &serv);
 
@@ -1294,7 +1338,6 @@ int main(int argc, char **argv) {
 
   io_loop_run(&loop, &serv);
 
-  nghttp2_option_del(shared_option);
   nghttp2_session_callbacks_del(shared_callbacks);
 
   return 0;
