@@ -31,19 +31,24 @@
 
 #include <nghttp2/nghttp2.h>
 #include "nghttp2_int.h"
+#include "nghttp2_mem.h"
 
 /* Implementation of priority queue */
 
+typedef struct { size_t index; } nghttp2_pq_entry;
+
 typedef struct {
   /* The pointer to the pointer to the item stored */
-  void **q;
+  nghttp2_pq_entry **q;
+  /* Memory allocator */
+  nghttp2_mem *mem;
   /* The number of items sotred */
   size_t length;
   /* The maximum number of items this pq can store. This is
      automatically extended when length is reached to this value. */
   size_t capacity;
-  /* The compare function between items */
-  nghttp2_compar compar;
+  /* The less function between items */
+  nghttp2_less less;
 } nghttp2_pq;
 
 /*
@@ -55,7 +60,7 @@ typedef struct {
  * NGHTTP2_ERR_NOMEM
  *     Out of memory.
  */
-int nghttp2_pq_init(nghttp2_pq *pq, nghttp2_compar cmp);
+int nghttp2_pq_init(nghttp2_pq *pq, nghttp2_less less, nghttp2_mem *mem);
 
 /*
  * Deallocates any resources allocated for |pq|.  The stored items are
@@ -72,13 +77,13 @@ void nghttp2_pq_free(nghttp2_pq *pq);
  * NGHTTP2_ERR_NOMEM
  *     Out of memory.
  */
-int nghttp2_pq_push(nghttp2_pq *pq, void *item);
+int nghttp2_pq_push(nghttp2_pq *pq, nghttp2_pq_entry *item);
 
 /*
  * Returns item at the top of the queue |pq|. If the queue is empty,
  * this function returns NULL.
  */
-void *nghttp2_pq_top(nghttp2_pq *pq);
+nghttp2_pq_entry *nghttp2_pq_top(nghttp2_pq *pq);
 
 /*
  * Pops item at the top of the queue |pq|. The popped item is not
@@ -96,7 +101,7 @@ int nghttp2_pq_empty(nghttp2_pq *pq);
  */
 size_t nghttp2_pq_size(nghttp2_pq *pq);
 
-typedef int (*nghttp2_pq_item_cb)(void *item, void *arg);
+typedef int (*nghttp2_pq_item_cb)(nghttp2_pq_entry *item, void *arg);
 
 /*
  * Updates each item in |pq| using function |fun| and re-construct
@@ -106,4 +111,18 @@ typedef int (*nghttp2_pq_item_cb)(void *item, void *arg);
  */
 void nghttp2_pq_update(nghttp2_pq *pq, nghttp2_pq_item_cb fun, void *arg);
 
+/*
+ * Applys |fun| to each item in |pq|.  The |arg| is passed as arg
+ * parameter to callback function.  This function must not change the
+ * ordering key.  If the return value from callback is nonzero, this
+ * function returns 1 immediately without iterating remaining items.
+ * Otherwise this function returns 0.
+ */
+int nghttp2_pq_each(nghttp2_pq *pq, nghttp2_pq_item_cb fun, void *arg);
+
+/*
+ * Removes |item| from priority queue.
+ */
+void nghttp2_pq_remove(nghttp2_pq *pq, nghttp2_pq_entry *item);
+
 #endif /* NGHTTP2_PQ_H */
