@@ -32,6 +32,35 @@
 #include "nghttp2_helper.h"
 #include "nghttp2_priority_spec.h"
 
+/*
+ * Detects the dependency error, that is stream attempted to depend on
+ * itself.  If |stream_id| is -1, we use session->next_stream_id as
+ * stream ID.
+ *
+ * This function returns 0 if it succeeds, or one of the following
+ * error codes:
+ *
+ * NGHTTP2_ERR_INVALID_ARGUMENT
+ *   Stream attempted to depend on itself.
+ */
+static int detect_self_dependency(nghttp2_session *session, int32_t stream_id,
+                                  const nghttp2_priority_spec *pri_spec) {
+  assert(pri_spec);
+
+  if (stream_id == -1) {
+    if ((int32_t)session->next_stream_id == pri_spec->stream_id) {
+      return NGHTTP2_ERR_INVALID_ARGUMENT;
+    }
+    return 0;
+  }
+
+  if (stream_id == pri_spec->stream_id) {
+    return NGHTTP2_ERR_INVALID_ARGUMENT;
+  }
+
+  return 0;
+}
+
 /* This function takes ownership of |nva_copy|. Regardless of the
    return value, the caller must not free |nva_copy| after this
    function returns. */
@@ -40,8 +69,7 @@ static int32_t submit_headers_shared(nghttp2_session *session, uint8_t flags,
                                      const nghttp2_priority_spec *pri_spec,
                                      nghttp2_nv *nva_copy, size_t nvlen,
                                      const nghttp2_data_provider *data_prd,
-                                     void *stream_user_data,
-                                     uint8_t attach_stream) {
+                                     void *stream_user_data) {
   int rv;
   uint8_t flags_copy;
   nghttp2_outbound_item *item = NULL;
@@ -51,11 +79,6 @@ static int32_t submit_headers_shared(nghttp2_session *session, uint8_t flags,
 
   mem = &session->mem;
 
-  if (stream_id == 0) {
-    rv = NGHTTP2_ERR_INVALID_ARGUMENT;
-    goto fail;
-  }
-
   item = nghttp2_mem_malloc(mem, sizeof(nghttp2_outbound_item));
   if (item == NULL) {
     rv = NGHTTP2_ERR_NOMEM;
@@ -69,7 +92,6 @@ static int32_t submit_headers_shared(nghttp2_session *session, uint8_t flags,
   }
 
   item->aux_data.headers.stream_user_data = stream_user_data;
-  item->aux_data.headers.attach_stream = attach_stream;
 
   flags_copy =
       (uint8_t)((flags & (NGHTTP2_FLAG_END_STREAM | NGHTTP2_FLAG_PRIORITY)) |
@@ -122,8 +144,7 @@ static int32_t submit_headers_shared_nva(nghttp2_session *session,
                                          const nghttp2_priority_spec *pri_spec,
                                          const nghttp2_nv *nva, size_t nvlen,
                                          const nghttp2_data_provider *data_prd,
-                                         void *stream_user_data,
-                                         uint8_t attach_stream) {
+                                         void *stream_user_data) {
   int rv;
   nghttp2_nv *nva_copy;
   nghttp2_priority_spec copy_pri_spec;
@@ -144,15 +165,18 @@ static int32_t submit_headers_shared_nva(nghttp2_session *session,
   }
 
   return submit_headers_shared(session, flags, stream_id, &copy_pri_spec,
-                               nva_copy, nvlen, data_prd, stream_user_data,
-                               attach_stream);
+                               nva_copy, nvlen, data_prd, stream_user_data);
 }
 
 int nghttp2_submit_trailer(nghttp2_session *session, int32_t stream_id,
                            const nghttp2_nv *nva, size_t nvlen) {
+  if (stream_id <= 0) {
+    return NGHTTP2_ERR_INVALID_ARGUMENT;
+  }
+
   return (int)submit_headers_shared_nva(session, NGHTTP2_FLAG_END_STREAM,
-                                        stream_id, NULL, nva, nvlen, NULL, NULL,
-                                        0);
+                                        stream_id, NULL, nva, nvlen, NULL,
+                                        NULL);
 }
 
 int32_t nghttp2_submit_headers(nghttp2_session *session, uint8_t flags,
@@ -160,16 +184,31 @@ int32_t nghttp2_submit_headers(nghttp2_session *session, uint8_t flags,
                                const nghttp2_priority_spec *pri_spec,
                                const nghttp2_nv *nva, size_t nvlen,
                                void *stream_user_data) {
+  int rv;
+
+  if (stream_id == -1) {
+    if (session->server) {
+      return NGHTTP2_ERR_PROTO;
+    }
+  } else if (stream_id <= 0) {
+    return NGHTTP2_ERR_INVALID_ARGUMENT;
+  }
+
   flags &= NGHTTP2_FLAG_END_STREAM;
 
   if (pri_spec && !nghttp2_priority_spec_check_default(pri_spec)) {
+    rv = detect_self_dependency(session, stream_id, pri_spec);
+    if (rv != 0) {
+      return rv;
+    }
+
     flags |= NGHTTP2_FLAG_PRIORITY;
   } else {
     pri_spec = NULL;
   }
 
   return submit_headers_shared_nva(session, flags, stream_id, pri_spec, nva,
-                                   nvlen, NULL, stream_user_data, 0);
+                                   nvlen, NULL, stream_user_data);
 }
 
 int nghttp2_submit_ping(nghttp2_session *session, uint8_t flags _U_,
@@ -275,7 +314,7 @@ int32_t nghttp2_submit_push_promise(nghttp2_session *session, uint8_t flags _U_,
 
   mem = &session->mem;
 
-  if (stream_id == 0 || nghttp2_session_is_my_stream_id(session, stream_id)) {
+  if (stream_id <= 0 || nghttp2_session_is_my_stream_id(session, stream_id)) {
     return NGHTTP2_ERR_INVALID_ARGUMENT;
   }
 
@@ -390,15 +429,25 @@ int32_t nghttp2_submit_request(nghttp2_session *session,
                                const nghttp2_data_provider *data_prd,
                                void *stream_user_data) {
   uint8_t flags;
+  int rv;
 
-  if (pri_spec && nghttp2_priority_spec_check_default(pri_spec)) {
+  if (session->server) {
+    return NGHTTP2_ERR_PROTO;
+  }
+
+  if (pri_spec && !nghttp2_priority_spec_check_default(pri_spec)) {
+    rv = detect_self_dependency(session, -1, pri_spec);
+    if (rv != 0) {
+      return rv;
+    }
+  } else {
     pri_spec = NULL;
   }
 
   flags = set_request_flags(pri_spec, data_prd);
 
   return submit_headers_shared_nva(session, flags, -1, pri_spec, nva, nvlen,
-                                   data_prd, stream_user_data, 0);
+                                   data_prd, stream_user_data);
 }
 
 static uint8_t set_response_flags(const nghttp2_data_provider *data_prd) {
@@ -412,9 +461,19 @@ static uint8_t set_response_flags(const nghttp2_data_provider *data_prd) {
 int nghttp2_submit_response(nghttp2_session *session, int32_t stream_id,
                             const nghttp2_nv *nva, size_t nvlen,
                             const nghttp2_data_provider *data_prd) {
-  uint8_t flags = set_response_flags(data_prd);
+  uint8_t flags;
+
+  if (stream_id <= 0) {
+    return NGHTTP2_ERR_INVALID_ARGUMENT;
+  }
+
+  if (!session->server) {
+    return NGHTTP2_ERR_PROTO;
+  }
+
+  flags = set_response_flags(data_prd);
   return submit_headers_shared_nva(session, flags, stream_id, NULL, nva, nvlen,
-                                   data_prd, NULL, 1);
+                                   data_prd, NULL);
 }
 
 int nghttp2_submit_data(nghttp2_session *session, uint8_t flags,
