@@ -33,22 +33,13 @@
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
-
-#include "asio_server.h"
-#include <stdexcept>
-#include <future>
-#include <boost/thread/thread.hpp>
-#include <boost/bind.hpp>
+#include "asio_io_service_pool.h"
 
 namespace nghttp2 {
 
 namespace asio_http2 {
 
-namespace server {
-
-io_service_pool::io_service_pool(std::size_t pool_size,
-                                 std::size_t thread_pool_size)
-    : next_io_service_(0), thread_pool_size_(thread_pool_size) {
+io_service_pool::io_service_pool(std::size_t pool_size) : next_io_service_(0) {
   if (pool_size == 0) {
     throw std::runtime_error("io_service_pool size is 0");
   }
@@ -61,32 +52,27 @@ io_service_pool::io_service_pool(std::size_t pool_size,
     io_services_.push_back(io_service);
     work_.push_back(work);
   }
-
-  auto work = std::make_shared<boost::asio::io_service::work>(task_io_service_);
-  work_.push_back(work);
 }
 
-void io_service_pool::run() {
-  for (std::size_t i = 0; i < thread_pool_size_; ++i) {
-    thread_pool_.create_thread([this]() { task_io_service_.run(); });
-  }
-
+void io_service_pool::run(bool asynchronous) {
   // Create a pool of threads to run all of the io_services.
-  auto futs = std::vector<std::future<std::size_t>>();
-
   for (std::size_t i = 0; i < io_services_.size(); ++i) {
-    futs.push_back(std::async(std::launch::async,
-                              (size_t (boost::asio::io_service::*)(void)) &
-                                  boost::asio::io_service::run,
-                              io_services_[i]));
+    futures_.push_back(std::async(std::launch::async,
+                                  (size_t (boost::asio::io_service::*)(void)) &
+                                      boost::asio::io_service::run,
+                                  io_services_[i]));
   }
 
+  if (!asynchronous) {
+    join();
+  }
+}
+
+void io_service_pool::join() {
   // Wait for all threads in the pool to exit.
-  for (auto &fut : futs) {
+  for (auto &fut : futures_) {
     fut.get();
   }
-
-  thread_pool_.join_all();
 }
 
 void io_service_pool::stop() {
@@ -94,8 +80,6 @@ void io_service_pool::stop() {
   for (auto &iosv : io_services_) {
     iosv->stop();
   }
-
-  task_io_service_.stop();
 }
 
 boost::asio::io_service &io_service_pool::get_io_service() {
@@ -108,12 +92,11 @@ boost::asio::io_service &io_service_pool::get_io_service() {
   return io_service;
 }
 
-boost::asio::io_service &io_service_pool::get_task_io_service() {
-  return task_io_service_;
+const std::vector<std::shared_ptr<boost::asio::io_service>> &
+io_service_pool::io_services() const {
+  return io_services_;
 }
 
-} // namespace server
-
 } // namespace asio_http2
 
 } // namespace nghttp2
