@@ -25,10 +25,11 @@
 #include "h2load_http2_session.h"
 
 #include <cassert>
+#include <cerrno>
 
 #include "h2load.h"
 #include "util.h"
-#include "libevent_util.h"
+#include "template.h"
 
 using namespace nghttp2;
 
@@ -50,6 +51,7 @@ int on_header_callback(nghttp2_session *session, const nghttp2_frame *frame,
     return 0;
   }
   client->on_header(frame->hd.stream_id, name, namelen, value, valuelen);
+  client->worker->stats.bytes_head_decomp += namelen + valuelen;
   return 0;
 }
 } // namespace
@@ -62,7 +64,12 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
       frame->headers.cat != NGHTTP2_HCAT_RESPONSE) {
     return 0;
   }
-  client->worker->stats.bytes_head += frame->hd.length;
+  client->worker->stats.bytes_head +=
+      frame->hd.length - frame->headers.padlen -
+      ((frame->hd.flags & NGHTTP2_FLAG_PRIORITY) ? 5 : 0);
+  if (frame->hd.flags & NGHTTP2_FLAG_END_STREAM) {
+    client->record_ttfb();
+  }
   return 0;
 }
 } // namespace
@@ -72,6 +79,7 @@ int on_data_chunk_recv_callback(nghttp2_session *session, uint8_t flags,
                                 int32_t stream_id, const uint8_t *data,
                                 size_t len, void *user_data) {
   auto client = static_cast<Client *>(user_data);
+  client->record_ttfb();
   client->worker->stats.bytes_body += len;
   return 0;
 }
@@ -82,10 +90,88 @@ int on_stream_close_callback(nghttp2_session *session, int32_t stream_id,
                              uint32_t error_code, void *user_data) {
   auto client = static_cast<Client *>(user_data);
   client->on_stream_close(stream_id, error_code == NGHTTP2_NO_ERROR);
+
   return 0;
 }
 } // namespace
 
+namespace {
+int on_frame_not_send_callback(nghttp2_session *session,
+                               const nghttp2_frame *frame, int lib_error_code,
+                               void *user_data) {
+  if (frame->hd.type != NGHTTP2_HEADERS ||
+      frame->headers.cat != NGHTTP2_HCAT_REQUEST) {
+    return 0;
+  }
+
+  auto client = static_cast<Client *>(user_data);
+  // request was not sent.  Mark it as error.
+  client->on_stream_close(frame->hd.stream_id, false);
+
+  return 0;
+}
+} // namespace
+
+namespace {
+int before_frame_send_callback(nghttp2_session *session,
+                               const nghttp2_frame *frame, void *user_data) {
+  if (frame->hd.type != NGHTTP2_HEADERS ||
+      frame->headers.cat != NGHTTP2_HCAT_REQUEST) {
+    return 0;
+  }
+
+  auto client = static_cast<Client *>(user_data);
+  auto req_stat = client->get_req_stat(frame->hd.stream_id);
+  assert(req_stat);
+  client->record_request_time(req_stat);
+
+  return 0;
+}
+} // namespace
+
+namespace {
+ssize_t file_read_callback(nghttp2_session *session, int32_t stream_id,
+                           uint8_t *buf, size_t length, uint32_t *data_flags,
+                           nghttp2_data_source *source, void *user_data) {
+  auto client = static_cast<Client *>(user_data);
+  auto config = client->worker->config;
+  auto req_stat = client->get_req_stat(stream_id);
+  assert(req_stat);
+  ssize_t nread;
+  while ((nread = pread(config->data_fd, buf, length, req_stat->data_offset)) ==
+             -1 &&
+         errno == EINTR)
+    ;
+
+  if (nread == -1) {
+    return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
+  }
+
+  req_stat->data_offset += nread;
+
+  if (nread == 0 || req_stat->data_offset == config->data_length) {
+    *data_flags |= NGHTTP2_DATA_FLAG_EOF;
+  }
+
+  return nread;
+}
+
+} // namespace
+
+namespace {
+ssize_t send_callback(nghttp2_session *session, const uint8_t *data,
+                      size_t length, int flags, void *user_data) {
+  auto client = static_cast<Client *>(user_data);
+  auto &wb = client->wb;
+
+  if (wb.wleft() == 0) {
+    return NGHTTP2_ERR_WOULDBLOCK;
+  }
+
+  return wb.write(data, length);
+}
+} // namespace
+
 void Http2Session::on_connect() {
   int rv;
 
@@ -93,8 +179,7 @@ void Http2Session::on_connect() {
 
   nghttp2_session_callbacks_new(&callbacks);
 
-  auto callbacks_deleter =
-      util::defer(callbacks, nghttp2_session_callbacks_del);
+  auto callbacks_deleter = defer(nghttp2_session_callbacks_del, callbacks);
 
   nghttp2_session_callbacks_set_on_frame_recv_callback(callbacks,
                                                        on_frame_recv_callback);
@@ -108,16 +193,24 @@ void Http2Session::on_connect() {
   nghttp2_session_callbacks_set_on_header_callback(callbacks,
                                                    on_header_callback);
 
+  nghttp2_session_callbacks_set_on_frame_not_send_callback(
+      callbacks, on_frame_not_send_callback);
+
+  nghttp2_session_callbacks_set_before_frame_send_callback(
+      callbacks, before_frame_send_callback);
+
+  nghttp2_session_callbacks_set_send_callback(callbacks, send_callback);
+
   nghttp2_session_client_new(&session_, callbacks, client_);
 
-  nghttp2_settings_entry iv[2];
+  std::array<nghttp2_settings_entry, 2> iv;
   iv[0].settings_id = NGHTTP2_SETTINGS_ENABLE_PUSH;
   iv[0].value = 0;
   iv[1].settings_id = NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE;
   iv[1].value = (1 << client_->worker->config->window_bits) - 1;
 
-  rv = nghttp2_submit_settings(session_, NGHTTP2_FLAG_NONE, iv,
-                               util::array_size(iv));
+  rv = nghttp2_submit_settings(session_, NGHTTP2_FLAG_NONE, iv.data(),
+                               iv.size());
 
   assert(rv == 0);
 
@@ -129,11 +222,14 @@ void Http2Session::on_connect() {
                                  extra_connection_window);
   }
 
-  bufferevent_write(client_->bev, NGHTTP2_CLIENT_CONNECTION_PREFACE,
-                    NGHTTP2_CLIENT_CONNECTION_PREFACE_LEN);
+  client_->signal_write();
 }
 
-void Http2Session::submit_request() {
+int Http2Session::submit_request() {
+  if (nghttp2_session_check_request_allowed(session_) == 0) {
+    return -1;
+  }
+
   auto config = client_->worker->config;
   auto &nva = config->nva[client_->reqidx++];
 
@@ -141,73 +237,49 @@ void Http2Session::submit_request() {
     client_->reqidx = 0;
   }
 
-  auto stream_id = nghttp2_submit_request(session_, nullptr, nva.data(),
-                                          nva.size(), nullptr, nullptr);
-  assert(stream_id > 0);
+  nghttp2_data_provider prd{{0}, file_read_callback};
+
+  auto stream_id =
+      nghttp2_submit_request(session_, nullptr, nva.data(), nva.size(),
+                             config->data_fd == -1 ? nullptr : &prd, nullptr);
+  if (stream_id < 0) {
+    return -1;
+  }
 
   client_->on_request(stream_id);
+
+  return 0;
 }
 
-ssize_t Http2Session::on_read() {
-  int rv;
-  size_t nread = 0;
-
-  auto input = bufferevent_get_input(client_->bev);
-
-  for (;;) {
-    auto inputlen = evbuffer_get_contiguous_space(input);
-
-    if (inputlen == 0) {
-      assert(evbuffer_get_length(input) == 0);
-
-      return nread;
-    }
-
-    auto mem = evbuffer_pullup(input, inputlen);
-
-    rv = nghttp2_session_mem_recv(session_, mem, inputlen);
-
-    if (rv < 0) {
-      return -1;
-    }
-
-    nread += rv;
-
-    if (evbuffer_drain(input, rv) != 0) {
-      return -1;
-    }
+int Http2Session::on_read(const uint8_t *data, size_t len) {
+  auto rv = nghttp2_session_mem_recv(session_, data, len);
+  if (rv < 0) {
+    return -1;
   }
+
+  assert(static_cast<size_t>(rv) == len);
+
+  if (nghttp2_session_want_read(session_) == 0 &&
+      nghttp2_session_want_write(session_) == 0 && client_->wb.rleft() == 0) {
+    return -1;
+  }
+
+  client_->signal_write();
+
+  return 0;
 }
 
 int Http2Session::on_write() {
-  int rv;
-  uint8_t buf[16384];
-  auto output = bufferevent_get_output(client_->bev);
-  util::EvbufferBuffer evbbuf(output, buf, sizeof(buf));
-  for (;;) {
-    const uint8_t *data;
-    auto datalen = nghttp2_session_mem_send(session_, &data);
-
-    if (datalen < 0) {
-      return -1;
-    }
-    if (datalen == 0) {
-      break;
-    }
-    rv = evbbuf.add(data, datalen);
-    if (rv != 0) {
-      return -1;
-    }
-  }
-  rv = evbbuf.flush();
+  auto rv = nghttp2_session_send(session_);
   if (rv != 0) {
     return -1;
   }
+
   if (nghttp2_session_want_read(session_) == 0 &&
-      nghttp2_session_want_write(session_) == 0 &&
-      evbuffer_get_length(output) == 0) {
+      nghttp2_session_want_write(session_) == 0 && client_->wb.rleft() == 0) {
     return -1;
   }
+
   return 0;
 }
 
