@@ -38,8 +38,8 @@ public:
   Http2Session(Client *client);
   virtual ~Http2Session();
   virtual void on_connect();
-  virtual void submit_request();
-  virtual ssize_t on_read();
+  virtual int submit_request();
+  virtual int on_read(const uint8_t *data, size_t len);
   virtual int on_write();
   virtual void terminate();
 
