@@ -25,6 +25,7 @@
 #include "h2load_spdy_session.h"
 
 #include <cassert>
+#include <cerrno>
 
 #include "h2load.h"
 #include "util.h"
@@ -47,6 +48,8 @@ void before_ctrl_send_callback(spdylay_session *session,
     return;
   }
   client->on_request(frame->syn_stream.stream_id);
+  auto req_stat = client->get_req_stat(frame->syn_stream.stream_id);
+  client->record_request_time(req_stat);
 }
 } // namespace
 
@@ -60,11 +63,22 @@ void on_ctrl_recv_callback(spdylay_session *session, spdylay_frame_type type,
   for (auto p = frame->syn_reply.nv; *p; p += 2) {
     auto name = *p;
     auto value = *(p + 1);
+    auto namelen = strlen(name);
+    auto valuelen = strlen(value);
     client->on_header(frame->syn_reply.stream_id,
-                      reinterpret_cast<const uint8_t *>(name), strlen(name),
-                      reinterpret_cast<const uint8_t *>(value), strlen(value));
+                      reinterpret_cast<const uint8_t *>(name), namelen,
+                      reinterpret_cast<const uint8_t *>(value), valuelen);
+    client->worker->stats.bytes_head_decomp += namelen + valuelen;
+  }
+
+  // Strictly speaking, we have to subtract 2 (unused field) if SPDY
+  // version is 2.  But it is already deprecated, and we don't do
+  // extra work for it.
+  client->worker->stats.bytes_head += frame->syn_reply.hd.length - 4;
+
+  if (frame->syn_stream.hd.flags & SPDYLAY_CTRL_FLAG_FIN) {
+    client->record_ttfb();
   }
-  client->worker->stats.bytes_head += frame->syn_reply.hd.length;
 }
 } // namespace
 
@@ -73,6 +87,8 @@ void on_data_chunk_recv_callback(spdylay_session *session, uint8_t flags,
                                  int32_t stream_id, const uint8_t *data,
                                  size_t len, void *user_data) {
   auto client = static_cast<Client *>(user_data);
+
+  client->record_ttfb();
   client->worker->stats.bytes_body += len;
 
   auto spdy_session = static_cast<SpdySession *>(client->session.get());
@@ -94,14 +110,41 @@ namespace {
 ssize_t send_callback(spdylay_session *session, const uint8_t *data,
                       size_t length, int flags, void *user_data) {
   auto client = static_cast<Client *>(user_data);
-  auto spdy_session = static_cast<SpdySession *>(client->session.get());
-  int rv;
+  auto &wb = client->wb;
 
-  rv = spdy_session->sendbuf.add(data, length);
-  if (rv != 0) {
-    return SPDYLAY_ERR_CALLBACK_FAILURE;
+  if (wb.wleft() == 0) {
+    return SPDYLAY_ERR_DEFERRED;
   }
-  return length;
+
+  return wb.write(data, length);
+}
+} // namespace
+
+namespace {
+ssize_t file_read_callback(spdylay_session *session, int32_t stream_id,
+                           uint8_t *buf, size_t length, int *eof,
+                           spdylay_data_source *source, void *user_data) {
+  auto client = static_cast<Client *>(user_data);
+  auto config = client->worker->config;
+  auto req_stat = client->get_req_stat(stream_id);
+
+  ssize_t nread;
+  while ((nread = pread(config->data_fd, buf, length, req_stat->data_offset)) ==
+             -1 &&
+         errno == EINTR)
+    ;
+
+  if (nread == -1) {
+    return SPDYLAY_ERR_TEMPORAL_CALLBACK_FAILURE;
+  }
+
+  req_stat->data_offset += nread;
+
+  if (nread == 0 || req_stat->data_offset == config->data_length) {
+    *eof = 1;
+  }
+
+  return nread;
 }
 } // namespace
 
@@ -119,12 +162,11 @@ void SpdySession::on_connect() {
   spdylay_session_set_option(session_, SPDYLAY_OPT_NO_AUTO_WINDOW_UPDATE, &val,
                              sizeof(val));
 
-  spdylay_settings_entry iv[1];
-  iv[0].settings_id = SPDYLAY_SETTINGS_INITIAL_WINDOW_SIZE;
-  iv[0].flags = SPDYLAY_ID_FLAG_SETTINGS_NONE;
-  iv[0].value = (1 << client_->worker->config->window_bits);
-  spdylay_submit_settings(session_, SPDYLAY_FLAG_SETTINGS_NONE, iv,
-                          util::array_size(iv));
+  spdylay_settings_entry iv;
+  iv.settings_id = SPDYLAY_SETTINGS_INITIAL_WINDOW_SIZE;
+  iv.flags = SPDYLAY_ID_FLAG_SETTINGS_NONE;
+  iv.value = (1 << client_->worker->config->window_bits);
+  spdylay_submit_settings(session_, SPDYLAY_FLAG_SETTINGS_NONE, &iv, 1);
 
   auto config = client_->worker->config;
 
@@ -134,9 +176,12 @@ void SpdySession::on_connect() {
         (1 << config->connection_window_bits) - SPDYLAY_INITIAL_WINDOW_SIZE;
     spdylay_submit_window_update(session_, 0, delta);
   }
+
+  client_->signal_write();
 }
 
-void SpdySession::submit_request() {
+int SpdySession::submit_request() {
+  int rv;
   auto config = client_->worker->config;
   auto &nv = config->nv[client_->reqidx++];
 
@@ -144,58 +189,44 @@ void SpdySession::submit_request() {
     client_->reqidx = 0;
   }
 
-  spdylay_submit_request(session_, 0, nv.data(), nullptr, nullptr);
-}
+  spdylay_data_provider prd{{0}, file_read_callback};
 
-ssize_t SpdySession::on_read() {
-  int rv;
-  size_t nread = 0;
-  auto input = bufferevent_get_input(client_->bev);
+  rv = spdylay_submit_request(session_, 0, nv.data(),
+                              config->data_fd == -1 ? nullptr : &prd, nullptr);
 
-  for (;;) {
-    auto inputlen = evbuffer_get_contiguous_space(input);
-
-    if (inputlen == 0) {
-      assert(evbuffer_get_length(input) == 0);
-
-      return nread;
-    }
-
-    auto mem = evbuffer_pullup(input, inputlen);
-
-    rv = spdylay_session_mem_recv(session_, mem, inputlen);
-
-    if (rv < 0) {
-      return -1;
-    }
-
-    nread += rv;
-
-    if (evbuffer_drain(input, rv) != 0) {
-      return -1;
-    }
-  }
-}
-
-int SpdySession::on_write() {
-  int rv;
-  uint8_t buf[16384];
-
-  sendbuf.reset(bufferevent_get_output(client_->bev), buf, sizeof(buf));
-
-  rv = spdylay_session_send(session_);
   if (rv != 0) {
     return -1;
   }
 
-  rv = sendbuf.flush();
+  return 0;
+}
+
+int SpdySession::on_read(const uint8_t *data, size_t len) {
+  auto rv = spdylay_session_mem_recv(session_, data, len);
+  if (rv < 0) {
+    return -1;
+  }
+
+  assert(static_cast<size_t>(rv) == len);
+
+  if (spdylay_session_want_read(session_) == 0 &&
+      spdylay_session_want_write(session_) == 0 && client_->wb.rleft() == 0) {
+    return -1;
+  }
+
+  client_->signal_write();
+
+  return 0;
+}
+
+int SpdySession::on_write() {
+  auto rv = spdylay_session_send(session_);
   if (rv != 0) {
     return -1;
   }
 
   if (spdylay_session_want_read(session_) == 0 &&
-      spdylay_session_want_write(session_) == 0 &&
-      evbuffer_get_length(bufferevent_get_output(client_->bev)) == 0) {
+      spdylay_session_want_write(session_) == 0 && client_->wb.rleft() == 0) {
     return -1;
   }
   return 0;
