@@ -25,18 +25,30 @@
 #ifndef SHRPX_HTTP2_TEST_H
 #define SHRPX_HTTP2_TEST_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
 namespace shrpx {
 
 void test_http2_add_header(void);
-void test_http2_check_http2_headers(void);
-void test_http2_get_unique_header(void);
 void test_http2_get_header(void);
-void test_http2_value_lws(void);
-void test_http2_copy_norm_headers_to_nva(void);
-void test_http2_build_http1_headers_from_norm_headers(void);
+void test_http2_copy_headers_to_nva(void);
+void test_http2_build_http1_headers_from_headers(void);
 void test_http2_lws(void);
 void test_http2_rewrite_location_uri(void);
 void test_http2_parse_http_status_code(void);
+void test_http2_index_header(void);
+void test_http2_lookup_token(void);
+void test_http2_check_http2_pseudo_header(void);
+void test_http2_http2_header_allowed(void);
+void test_http2_mandatory_request_headers_presence(void);
+void test_http2_parse_link_header(void);
+void test_http2_path_join(void);
+void test_http2_normalize_path(void);
+void test_http2_rewrite_clean_path(void);
+void test_http2_get_pure_path_component(void);
+void test_http2_construct_push_component(void);
 
 } // namespace shrpx
 
