@@ -26,7 +26,9 @@
 #include <config.h>
 #endif // HAVE_CONFIG_H
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif // HAVE_UNISTD_H
 #include <getopt.h>
 
 #include <cstdio>
@@ -39,13 +41,12 @@
 
 #include <jansson.h>
 
-extern "C" {
-
-#include "nghttp2_hd.h"
-#include "nghttp2_frame.h"
+#include <nghttp2/nghttp2.h>
 
+#include "template.h"
 #include "comp_helper.h"
-}
+
+namespace nghttp2 {
 
 typedef struct { int dump_header_table; } inflate_config;
 
@@ -74,12 +75,15 @@ static void to_json(nghttp2_hd_inflater *inflater, json_t *headers,
   json_object_set_new(obj, "seq", json_integer(seq));
   json_object_set(obj, "wire", wire);
   json_object_set(obj, "headers", headers);
-  if (old_settings_table_size != inflater->settings_hd_table_bufsize_max) {
+  auto max_dyn_table_size =
+      nghttp2_hd_inflate_get_max_dynamic_table_size(inflater);
+  if (old_settings_table_size != max_dyn_table_size) {
     json_object_set_new(obj, "header_table_size",
-                        json_integer(inflater->settings_hd_table_bufsize_max));
+                        json_integer(max_dyn_table_size));
   }
   if (config.dump_header_table) {
-    json_object_set_new(obj, "header_table", dump_header_table(&inflater->ctx));
+    json_object_set_new(obj, "header_table",
+                        dump_inflate_header_table(inflater));
   }
   json_dumpf(obj, stdout, JSON_INDENT(2) | JSON_PRESERVE_ORDER);
   json_decref(obj);
@@ -90,7 +94,8 @@ static int inflate_hd(json_t *obj, nghttp2_hd_inflater *inflater, int seq) {
   ssize_t rv;
   nghttp2_nv nv;
   int inflate_flags;
-  size_t old_settings_table_size = inflater->settings_hd_table_bufsize_max;
+  size_t old_settings_table_size =
+      nghttp2_hd_inflate_get_max_dynamic_table_size(inflater);
 
   auto wire = json_object_get(obj, "wire");
 
@@ -99,6 +104,11 @@ static int inflate_hd(json_t *obj, nghttp2_hd_inflater *inflater, int seq) {
     return -1;
   }
 
+  if (!json_is_string(wire)) {
+    fprintf(stderr, "'wire' value is not string at %d\n", seq);
+    return -1;
+  }
+
   auto table_size = json_object_get(obj, "header_table_size");
 
   if (table_size) {
@@ -159,7 +169,7 @@ static int inflate_hd(json_t *obj, nghttp2_hd_inflater *inflater, int seq) {
 }
 
 static int perform(void) {
-  nghttp2_hd_inflater inflater;
+  nghttp2_hd_inflater *inflater = NULL;
   json_error_t error;
 
   auto json = json_loadf(stdin, 0, &error);
@@ -181,7 +191,7 @@ static int perform(void) {
     exit(EXIT_FAILURE);
   }
 
-  nghttp2_hd_inflate_init(&inflater);
+  nghttp2_hd_inflate_new(&inflater);
   output_json_header();
   auto len = json_array_size(cases);
 
@@ -191,7 +201,7 @@ static int perform(void) {
       fprintf(stderr, "Unexpected JSON type at %zu. It should be object.\n", i);
       continue;
     }
-    if (inflate_hd(obj, &inflater, i) != 0) {
+    if (inflate_hd(obj, inflater, i) != 0) {
       continue;
     }
     if (i + 1 < len) {
@@ -199,7 +209,7 @@ static int perform(void) {
     }
   }
   output_json_footer();
-  nghttp2_hd_inflate_free(&inflater);
+  nghttp2_hd_inflate_del(inflater);
   json_decref(json);
 
   return 0;
@@ -268,3 +278,9 @@ int main(int argc, char **argv) {
   perform();
   return 0;
 }
+
+} // namespace nghttp2
+
+int main(int argc, char **argv) {
+  return nghttp2::run_app(nghttp2::main, argc, argv);
+}
