@@ -26,11 +26,10 @@
 
 #include <assert.h>
 
-int nghttp2_gzip_inflate_new(nghttp2_gzip **inflater_ptr)
-{
+int nghttp2_gzip_inflate_new(nghttp2_gzip **inflater_ptr) {
   int rv;
   *inflater_ptr = malloc(sizeof(nghttp2_gzip));
-  if(*inflater_ptr == NULL) {
+  if (*inflater_ptr == NULL) {
     return -1;
   }
   (*inflater_ptr)->finished = 0;
@@ -40,31 +39,29 @@ int nghttp2_gzip_inflate_new(nghttp2_gzip **inflater_ptr)
   (*inflater_ptr)->zst.zfree = Z_NULL;
   (*inflater_ptr)->zst.opaque = Z_NULL;
   rv = inflateInit2(&(*inflater_ptr)->zst, 47);
-  if(rv != Z_OK) {
+  if (rv != Z_OK) {
     free(*inflater_ptr);
     return -1;
   }
   return 0;
 }
 
-void nghttp2_gzip_inflate_del(nghttp2_gzip *inflater)
-{
-  if(inflater != NULL) {
+void nghttp2_gzip_inflate_del(nghttp2_gzip *inflater) {
+  if (inflater != NULL) {
     inflateEnd(&inflater->zst);
     free(inflater);
   }
 }
 
-int nghttp2_gzip_inflate(nghttp2_gzip *inflater,
-                         uint8_t *out, size_t *outlen_ptr,
-                         const uint8_t *in, size_t *inlen_ptr)
-{
+int nghttp2_gzip_inflate(nghttp2_gzip *inflater, uint8_t *out,
+                         size_t *outlen_ptr, const uint8_t *in,
+                         size_t *inlen_ptr) {
   int rv;
-  if(inflater->finished) {
+  if (inflater->finished) {
     return -1;
   }
   inflater->zst.avail_in = (unsigned int)*inlen_ptr;
-  inflater->zst.next_in = (unsigned char*)in;
+  inflater->zst.next_in = (unsigned char *)in;
   inflater->zst.avail_out = (unsigned int)*outlen_ptr;
   inflater->zst.next_out = out;
 
@@ -72,7 +69,7 @@ int nghttp2_gzip_inflate(nghttp2_gzip *inflater,
 
   *inlen_ptr -= inflater->zst.avail_in;
   *outlen_ptr -= inflater->zst.avail_out;
-  switch(rv) {
+  switch (rv) {
   case Z_STREAM_END:
     inflater->finished = 1;
   case Z_OK:
@@ -90,7 +87,6 @@ int nghttp2_gzip_inflate(nghttp2_gzip *inflater,
   }
 }
 
-int nghttp2_gzip_inflate_finished(nghttp2_gzip *inflater)
-{
+int nghttp2_gzip_inflate_finished(nghttp2_gzip *inflater) {
   return inflater->finished;
 }
