@@ -25,6 +25,10 @@
 #ifndef NGHTTP2_GZIP_TEST_H
 #define NGHTTP2_GZIP_TEST_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
+
 #ifdef __cplusplus
 extern "C" {
 #endif
