@@ -160,6 +160,12 @@ Options:
               are used.
   --echo-upload
               Send back uploaded content if method is POST or PUT.
+  --mime-types-file=<PATH>
+              Path  to file  that contains  MIME media  types and  the
+              extensions that represent them.
+              Default: )" << config.mime_types_file << R"(
+  --no-content-length
+              Don't send content-length header field.
   --version   Display version information and exit.
   -h, --help  Display this help and exit.
 
@@ -179,6 +185,8 @@ int main(int argc, char **argv) {
 
   Config config;
   bool color = false;
+  auto mime_types_file_set_manually = false;
+
   while (1) {
     static int flag = 0;
     static option long_options[] = {
@@ -202,6 +210,8 @@ int main(int argc, char **argv) {
         {"trailer", required_argument, &flag, 6},
         {"hexdump", no_argument, &flag, 7},
         {"echo-upload", no_argument, &flag, 8},
+        {"mime-types-file", required_argument, &flag, 9},
+        {"no-content-length", no_argument, &flag, 10},
         {nullptr, 0, nullptr, 0}};
     int option_index = 0;
     int c = getopt_long(argc, argv, "DVb:c:d:ehm:n:p:va:", long_options,
@@ -328,6 +338,15 @@ int main(int argc, char **argv) {
         // echo-upload option
         config.echo_upload = true;
         break;
+      case 9:
+        // mime-types-file option
+        mime_types_file_set_manually = true;
+        config.mime_types_file = optarg;
+        break;
+      case 10:
+        // no-content-length option
+        config.no_content_length = true;
+        break;
       }
       break;
     default:
@@ -366,6 +385,14 @@ int main(int argc, char **argv) {
     config.htdocs = "./";
   }
 
+  if (util::read_mime_types(config.mime_types,
+                            config.mime_types_file.c_str()) != 0) {
+    if (mime_types_file_set_manually) {
+      std::cerr << "--mime-types-file: Could not open mime types file: "
+                << config.mime_types_file << std::endl;
+    }
+  }
+
   set_color_output(color || isatty(fileno(stdout)));
 
   struct sigaction act {};
