@@ -30,26 +30,22 @@
 #endif // HAVE_CONFIG_H
 
 #include <sys/types.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif // HAVE_SYS_SOCKET_H
 
 #include <cassert>
 
 #include "shrpx_log.h"
 
-#ifndef SOCK_NONBLOCK
-#define SOCK_NONBLOCK 0
-#endif // !SOCK_NONBLOCK
-
-#ifndef SOCK_CLOEXEC
-#define SOCK_CLOEXEC 0
-#endif // !SOCK_CLOEXEC
-
 #ifndef HAVE__EXIT
 #define _Exit(status) _exit(status)
 #endif // !HAVE__EXIT
 
-#define DIE() exit(EXIT_FAILURE)
+#define DIE() _Exit(EXIT_FAILURE)
 
-#define SHRPX_READ_WATERMARK (16 * 1024)
+#if defined(HAVE_DECL_INITGROUPS) && !HAVE_DECL_INITGROUPS
+inline int initgroups(const char *user, gid_t group) { return 0; }
+#endif // defined(HAVE_DECL_INITGROUPS) && !HAVE_DECL_INITGROUPS
 
 #endif // SHRPX_H
