@@ -49,17 +49,160 @@ class Upstream;
 class DownstreamConnection;
 struct BlockedLink;
 
+class FieldStore {
+public:
+  FieldStore(size_t headers_initial_capacity)
+      : content_length(-1), buffer_size_(0), header_key_prev_(false),
+        trailer_key_prev_(false) {
+    http2::init_hdidx(hdidx_);
+    headers_.reserve(headers_initial_capacity);
+  }
+
+  const Headers &headers() const { return headers_; }
+  const Headers &trailers() const { return trailers_; }
+
+  Headers &headers() { return headers_; }
+
+  const void add_extra_buffer_size(size_t n) { buffer_size_ += n; }
+  size_t buffer_size() const { return buffer_size_; }
+
+  size_t num_fields() const { return headers_.size() + trailers_.size(); }
+
+  // Returns pointer to the header field with the name |name|.  If
+  // multiple header have |name| as name, return last occurrence from
+  // the beginning.  If no such header is found, returns nullptr.
+  // This function must be called after headers are indexed
+  const Headers::value_type *header(int16_t token) const;
+  Headers::value_type *header(int16_t token);
+  // Returns pointer to the header field with the name |name|.  If no
+  // such header is found, returns nullptr.
+  const Headers::value_type *header(const StringRef &name) const;
+
+  void add_header(std::string name, std::string value);
+  void add_header(std::string name, std::string value, int16_t token);
+  void add_header(const uint8_t *name, size_t namelen, const uint8_t *value,
+                  size_t valuelen, bool no_index, int16_t token);
+
+  void append_last_header_key(const char *data, size_t len);
+  void append_last_header_value(const char *data, size_t len);
+
+  bool header_key_prev() const { return header_key_prev_; }
+
+  // Lower the header field names and indexes header fields.  If there
+  // is any invalid headers (e.g., multiple Content-Length having
+  // different values), returns -1.
+  int index_headers();
+
+  // Empties headers.
+  void clear_headers();
+
+  void add_trailer(const uint8_t *name, size_t namelen, const uint8_t *value,
+                   size_t valuelen, bool no_index, int16_t token);
+  void add_trailer(std::string name, std::string value);
+
+  void append_last_trailer_key(const char *data, size_t len);
+  void append_last_trailer_value(const char *data, size_t len);
+
+  bool trailer_key_prev() const { return trailer_key_prev_; }
+
+  // content-length, -1 if it is unknown.
+  int64_t content_length;
+
+private:
+  Headers headers_;
+  // trailer fields.  For HTTP/1.1, trailer fields are only included
+  // with chunked encoding.  For HTTP/2, there is no such limit.
+  Headers trailers_;
+  http2::HeaderIndex hdidx_;
+  // Sum of the length of name and value in headers_ and trailers_.
+  // This could also be increased by add_extra_buffer_size() to take
+  // into account for request URI in case of HTTP/1.x request.
+  size_t buffer_size_;
+  bool header_key_prev_;
+  bool trailer_key_prev_;
+};
+
+struct Request {
+  Request()
+      : fs(16), recv_body_length(0), unconsumed_body_length(0), method(-1),
+        http_major(1), http_minor(1), upgrade_request(false),
+        http2_upgrade_seen(false), connection_close(false),
+        http2_expect_body(false), no_authority(false) {}
+
+  void consume(size_t len) {
+    assert(unconsumed_body_length >= len);
+    unconsumed_body_length -= len;
+  }
+
+  FieldStore fs;
+  // Request scheme.  For HTTP/2, this is :scheme header field value.
+  // For HTTP/1.1, this is deduced from URI or connection.
+  std::string scheme;
+  // Request authority.  This is HTTP/2 :authority header field value
+  // or host header field value.  We may deduce it from absolute-form
+  // HTTP/1 request.  We also store authority-form HTTP/1 request.
+  // This could be empty if request comes from HTTP/1.0 without Host
+  // header field and origin-form.
+  std::string authority;
+  // Request path, including query component.  For HTTP/1.1, this is
+  // request-target.  For HTTP/2, this is :path header field value.
+  // For CONNECT request, this is empty.
+  std::string path;
+  // the length of request body received so far
+  int64_t recv_body_length;
+  // The number of bytes not consumed by the application yet.
+  size_t unconsumed_body_length;
+  int method;
+  // HTTP major and minor version
+  int http_major, http_minor;
+  // Returns true if the request is HTTP upgrade (HTTP Upgrade or
+  // CONNECT method).  Upgrade to HTTP/2 is excluded.  For HTTP/2
+  // Upgrade, check get_http2_upgrade_request().
+  bool upgrade_request;
+  // true if h2c is seen in Upgrade header field.
+  bool http2_upgrade_seen;
+  bool connection_close;
+  // true if this is HTTP/2, and request body is expected.  Note that
+  // we don't take into account HTTP method here.
+  bool http2_expect_body;
+  // true if request does not have any information about authority.
+  // This happens when: For HTTP/2 request, :authority is missing.
+  // For HTTP/1 request, origin or asterisk form is used.
+  bool no_authority;
+};
+
+struct Response {
+  Response()
+      : fs(32), recv_body_length(0), unconsumed_body_length(0), http_status(0),
+        http_major(1), http_minor(1), connection_close(false) {}
+
+  void consume(size_t len) {
+    assert(unconsumed_body_length >= len);
+    unconsumed_body_length -= len;
+  }
+
+  FieldStore fs;
+  // the length of response body received so far
+  int64_t recv_body_length;
+  // The number of bytes not consumed by the application yet.  This is
+  // mainly for HTTP/2 backend.
+  size_t unconsumed_body_length;
+  // HTTP status code
+  unsigned int http_status;
+  int http_major, http_minor;
+  bool connection_close;
+};
+
 class Downstream {
 public:
-  Downstream(Upstream *upstream, MemchunkPool *mcpool, int32_t stream_id,
-             int32_t priority);
+  Downstream(Upstream *upstream, MemchunkPool *mcpool, int32_t stream_id);
   ~Downstream();
   void reset_upstream(Upstream *upstream);
   Upstream *get_upstream() const;
   void set_stream_id(int32_t stream_id);
   int32_t get_stream_id() const;
-  void set_priority(int32_t pri);
-  int32_t get_priority() const;
+  void set_assoc_stream_id(int32_t stream_id);
+  int32_t get_assoc_stream_id() const;
   void pause_read(IOCtrlReason reason);
   int resume_read(IOCtrlReason reason, size_t consumed);
   void force_resume_read();
@@ -79,9 +222,6 @@ public:
   // Returns true if upgrade (HTTP Upgrade or CONNECT) is succeeded.
   // This should not depend on inspect_http1_response().
   void check_upgrade_fulfilled();
-  // Returns true if the request is upgrade.  Upgrade to HTTP/2 is
-  // excluded.  For HTTP/2 Upgrade, check get_http2_upgrade_request().
-  bool get_upgrade_request() const;
   // Returns true if the upgrade is succeded as a result of the call
   // check_upgrade_fulfilled().  HTTP/2 Upgrade is excluded.
   bool get_upgraded() const;
@@ -94,97 +234,32 @@ public:
   bool get_http2_upgrade_request() const;
   // Returns the value of HTTP2-Settings request header field.
   const std::string &get_http2_settings() const;
+
   // downstream request API
-  const Headers &get_request_headers() const;
-  Headers &get_request_headers();
-  // Crumbles (split cookie by ";") in request_headers_ and returns
-  // them.  Headers::no_index is inherited.
-  Headers crumble_request_cookie();
-  void assemble_request_cookie();
-  const std::string &get_assembled_request_cookie() const;
-  // Lower the request header field names and indexes request headers.
-  // If there is any invalid headers (e.g., multiple Content-Length
-  // having different values), returns -1.
-  int index_request_headers();
-  // Returns pointer to the request header with the name |name|.  If
-  // multiple header have |name| as name, return last occurrence from
-  // the beginning.  If no such header is found, returns nullptr.
-  // This function must be called after headers are indexed
-  const Headers::value_type *get_request_header(int16_t token) const;
-  // Returns pointer to the request header with the name |name|.  If
-  // no such header is found, returns nullptr.
-  const Headers::value_type *get_request_header(const std::string &name) const;
-  void add_request_header(std::string name, std::string value);
-  void set_last_request_header_value(const char *data, size_t len);
+  const Request &request() const { return req_; }
+  Request &request() { return req_; }
 
-  void add_request_header(std::string name, std::string value, int16_t token);
-  void add_request_header(const uint8_t *name, size_t namelen,
-                          const uint8_t *value, size_t valuelen, bool no_index,
-                          int16_t token);
+  // Count number of crumbled cookies
+  size_t count_crumble_request_cookie();
+  // Crumbles (split cookie by ";") in request_headers_ and adds them
+  // in |nva|.  Headers::no_index is inherited.
+  void crumble_request_cookie(std::vector<nghttp2_nv> &nva);
+  // Assembles request cookies.  The opposite operation against
+  // crumble_request_cookie().
+  std::string assemble_request_cookie() const;
 
-  bool get_request_header_key_prev() const;
-  void append_last_request_header_key(const char *data, size_t len);
-  void append_last_request_header_value(const char *data, size_t len);
-  // Empties request headers.
-  void clear_request_headers();
-
-  size_t get_request_headers_sum() const;
-
-  const Headers &get_request_trailers() const;
-  void add_request_trailer(const uint8_t *name, size_t namelen,
-                           const uint8_t *value, size_t valuelen, bool no_index,
-                           int16_t token);
-  void add_request_trailer(std::string name, std::string value);
-  void set_last_request_trailer_value(const char *data, size_t len);
-  bool get_request_trailer_key_prev() const;
-  void append_last_request_trailer_key(const char *data, size_t len);
-  void append_last_request_trailer_value(const char *data, size_t len);
-
-  void set_request_method(int method);
-  int get_request_method() const;
-  void set_request_path(std::string path);
-  void add_request_headers_sum(size_t amount);
   void
   set_request_start_time(std::chrono::high_resolution_clock::time_point time);
   const std::chrono::high_resolution_clock::time_point &
   get_request_start_time() const;
-  void append_request_path(const char *data, size_t len);
-  // Returns request path. For HTTP/1.1, this is request-target. For
-  // HTTP/2, this is :path header field value.  For CONNECT request,
-  // this is empty.
-  const std::string &get_request_path() const;
-  // Returns HTTP/2 :scheme header field value.
-  const std::string &get_request_http2_scheme() const;
-  void set_request_http2_scheme(std::string scheme);
-  // Returns :authority or host header field value.  We may deduce it
-  // from absolute-form HTTP/1 request.  We also store authority-form
-  // HTTP/1 request.  This could be empty if request comes from
-  // HTTP/1.0 without Host header field and origin-form.
-  const std::string &get_request_http2_authority() const;
-  void set_request_http2_authority(std::string authority);
-  void append_request_http2_authority(const char *data, size_t len);
-  void set_request_major(int major);
-  void set_request_minor(int minor);
-  int get_request_major() const;
-  int get_request_minor() const;
   int push_request_headers();
   bool get_chunked_request() const;
   void set_chunked_request(bool f);
-  bool get_request_connection_close() const;
-  void set_request_connection_close(bool f);
-  bool get_request_http2_expect_body() const;
-  void set_request_http2_expect_body(bool f);
   int push_upload_data_chunk(const uint8_t *data, size_t datalen);
   int end_upload_data();
-  size_t get_request_datalen() const;
-  void dec_request_datalen(size_t len);
-  void reset_request_datalen();
   // Validates that received request body length and content-length
   // matches.
-  bool validate_request_bodylen() const;
-  int64_t get_request_content_length() const;
-  void set_request_content_length(int64_t len);
-  bool request_pseudo_header_allowed(int16_t token) const;
+  bool validate_request_recv_body_length() const;
   void set_request_downstream_host(std::string host);
   bool expect_response_body() const;
   enum {
@@ -210,71 +285,24 @@ public:
   bool get_request_pending() const;
   // Returns true if request is ready to be submitted to downstream.
   bool request_submission_ready() const;
+
   // downstream response API
-  const Headers &get_response_headers() const;
-  Headers &get_response_headers();
-  // Lower the response header field names and indexes response
-  // headers.  If there are invalid headers (e.g., multiple
-  // Content-Length with different values), returns -1.
-  int index_response_headers();
-  // Returns pointer to the response header with the name |name|.  If
-  // multiple header have |name| as name, return last occurrence from
-  // the beginning.  If no such header is found, returns nullptr.
-  // This function must be called after response headers are indexed.
-  const Headers::value_type *get_response_header(int16_t token) const;
-  Headers::value_type *get_response_header(int16_t token);
+  const Response &response() const { return resp_; }
+  Response &response() { return resp_; }
+
   // Rewrites the location response header field.
   void rewrite_location_response_header(const std::string &upstream_scheme);
-  void add_response_header(std::string name, std::string value);
-  void set_last_response_header_value(const char *data, size_t len);
 
-  void add_response_header(std::string name, std::string value, int16_t token);
-  void add_response_header(const uint8_t *name, size_t namelen,
-                           const uint8_t *value, size_t valuelen, bool no_index,
-                           int16_t token);
-
-  bool get_response_header_key_prev() const;
-  void append_last_response_header_key(const char *data, size_t len);
-  void append_last_response_header_value(const char *data, size_t len);
-  // Empties response headers.
-  void clear_response_headers();
-
-  size_t get_response_headers_sum() const;
-
-  const Headers &get_response_trailers() const;
-  void add_response_trailer(const uint8_t *name, size_t namelen,
-                            const uint8_t *value, size_t valuelen,
-                            bool no_index, int16_t token);
-  void add_response_trailer(std::string name, std::string value);
-  void set_last_response_trailer_value(const char *data, size_t len);
-  bool get_response_trailer_key_prev() const;
-  void append_last_response_trailer_key(const char *data, size_t len);
-  void append_last_response_trailer_value(const char *data, size_t len);
-
-  unsigned int get_response_http_status() const;
-  void set_response_http_status(unsigned int status);
-  void set_response_major(int major);
-  void set_response_minor(int minor);
-  int get_response_major() const;
-  int get_response_minor() const;
-  int get_response_version() const;
   bool get_chunked_response() const;
   void set_chunked_response(bool f);
-  bool get_response_connection_close() const;
-  void set_response_connection_close(bool f);
+
   void set_response_state(int state);
   int get_response_state() const;
   DefaultMemchunks *get_response_buf();
   bool response_buf_full();
-  void add_response_bodylen(size_t amount);
-  int64_t get_response_bodylen() const;
-  void add_response_sent_bodylen(size_t amount);
-  int64_t get_response_sent_bodylen() const;
-  int64_t get_response_content_length() const;
-  void set_response_content_length(int64_t len);
   // Validates that received response body length and content-length
   // matches.
-  bool validate_response_bodylen() const;
+  bool validate_response_recv_body_length() const;
   uint32_t get_response_rst_stream_error_code() const;
   void set_response_rst_stream_error_code(uint32_t error_code);
   // Inspects HTTP/1 response.  This checks tranfer-encoding etc.
@@ -286,22 +314,11 @@ public:
   bool get_non_final_response() const;
   void set_expect_final_response(bool f);
   bool get_expect_final_response() const;
-  void add_response_datalen(size_t len);
-  void dec_response_datalen(size_t len);
-  size_t get_response_datalen() const;
-  void reset_response_datalen();
-  bool response_pseudo_header_allowed(int16_t token) const;
 
   // Call this method when there is incoming data in downstream
   // connection.
   int on_read();
 
-  // Change the priority of downstream
-  int change_priority(int32_t pri);
-
-  bool get_rst_stream_after_end_stream() const;
-  void set_rst_stream_after_end_stream(bool f);
-
   // Resets upstream read timer.  If it is active, timeout value is
   // reset.  If it is not active, timer will be started.
   void reset_upstream_rtimer();
@@ -359,25 +376,19 @@ public:
 
   Downstream *dlnext, *dlprev;
 
-private:
-  Headers request_headers_;
-  Headers response_headers_;
+  // the length of response body sent to upstream client
+  int64_t response_sent_body_length;
 
-  // trailer part.  For HTTP/1.1, trailer part is only included with
-  // chunked encoding.  For HTTP/2, there is no such limit.
-  Headers request_trailers_;
-  Headers response_trailers_;
+private:
+  Request req_;
+  Response resp_;
 
   std::chrono::high_resolution_clock::time_point request_start_time_;
 
-  std::string request_path_;
-  std::string request_http2_scheme_;
-  std::string request_http2_authority_;
   // host we requested to downstream.  This is used to rewrite
   // location header field to decide the location should be rewritten
   // or not.
   std::string request_downstream_host_;
-  std::string assembled_request_cookie_;
 
   DefaultMemchunks request_buf_;
   DefaultMemchunks response_buf_;
@@ -388,76 +399,36 @@ private:
   ev_timer downstream_rtimer_;
   ev_timer downstream_wtimer_;
 
-  // the length of request body received so far
-  int64_t request_bodylen_;
-  // the length of response body received so far
-  int64_t response_bodylen_;
-
-  // the length of response body sent to upstream client
-  int64_t response_sent_bodylen_;
-
-  // content-length of request body, -1 if it is unknown.
-  int64_t request_content_length_;
-  // content-length of response body, -1 if it is unknown.
-  int64_t response_content_length_;
-
   Upstream *upstream_;
   std::unique_ptr<DownstreamConnection> dconn_;
 
   // only used by HTTP/2 or SPDY upstream
   BlockedLink *blocked_link_;
-
-  size_t request_headers_sum_;
-  size_t response_headers_sum_;
-
-  // The number of bytes not consumed by the application yet.
-  size_t request_datalen_;
-  size_t response_datalen_;
-
+  // How many times we tried in backend connection
   size_t num_retry_;
-
+  // The stream ID in frontend connection
   int32_t stream_id_;
-  int32_t priority_;
+  // The associated stream ID in frontend connection if this is pushed
+  // stream.
+  int32_t assoc_stream_id_;
   // stream ID in backend connection
   int32_t downstream_stream_id_;
-
   // RST_STREAM error_code from downstream HTTP2 connection
   uint32_t response_rst_stream_error_code_;
-
-  int request_method_;
+  // request state
   int request_state_;
-  int request_major_;
-  int request_minor_;
-
+  // response state
   int response_state_;
-  unsigned int response_http_status_;
-  int response_major_;
-  int response_minor_;
-
   // only used by HTTP/2 or SPDY upstream
   int dispatch_state_;
-
-  http2::HeaderIndex request_hdidx_;
-  http2::HeaderIndex response_hdidx_;
-
-  // true if the request contains upgrade token (HTTP Upgrade or
-  // CONNECT)
-  bool upgrade_request_;
-  // true if the connection is upgraded (HTTP Upgrade or CONNECT)
+  // true if the connection is upgraded (HTTP Upgrade or CONNECT),
+  // excluding upgrade to HTTP/2.
   bool upgraded_;
-
-  bool http2_upgrade_seen_;
-
+  // true if backend request uses chunked transfer-encoding
   bool chunked_request_;
-  bool request_connection_close_;
-  bool request_header_key_prev_;
-  bool request_trailer_key_prev_;
-  bool request_http2_expect_body_;
-
+  // true if response to client uses chunked transfer-encoding
   bool chunked_response_;
-  bool response_connection_close_;
-  bool response_header_key_prev_;
-  bool response_trailer_key_prev_;
+  // true if we have not got final response code
   bool expect_final_response_;
   // true if downstream request is pending because backend connection
   // has not been established or should be checked before use;
