@@ -26,7 +26,6 @@
 
 #include "shrpx_config.h"
 #include "shrpx_log.h"
-#include "shrpx_worker_config.h"
 #include "http2.h"
 #include "util.h"
 
@@ -40,28 +39,84 @@ std::string create_error_html(unsigned int status_code) {
   std::string res;
   res.reserve(512);
   auto status = http2::get_status_string(status_code);
-  res += "<html><head><title>";
+  res += R"(<!DOCTYPE html><html lang="en"><title>)";
   res += status;
-  res += "</title></head><body><h1>";
+  res += "</title><body><h1>";
   res += status;
-  res += "</h1><hr><address>";
-  res += get_config()->server_name;
+  res += "</h1><footer>";
+  const auto &server_name = get_config()->http.server_name;
+  res.append(server_name.c_str(), server_name.size());
   res += " at port ";
-  res += util::utos(get_config()->port);
-  res += "</address>";
-  res += "</body></html>";
+  res += util::utos(get_config()->conn.listener.port);
+  res += "</footer></body></html>";
   return res;
 }
 
 std::string create_via_header_value(int major, int minor) {
   std::string hdrs;
   hdrs += static_cast<char>(major + '0');
-  hdrs += ".";
-  hdrs += static_cast<char>(minor + '0');
+  if (major < 2) {
+    hdrs += '.';
+    hdrs += static_cast<char>(minor + '0');
+  }
   hdrs += " nghttpx";
   return hdrs;
 }
 
+std::string create_forwarded(int params, const std::string &node_by,
+                             const std::string &node_for,
+                             const std::string &host,
+                             const std::string &proto) {
+  std::string res;
+  if ((params & FORWARDED_BY) && !node_by.empty()) {
+    // This must be quoted-string unless it is obfuscated version
+    // (which starts with "_"), since ':' is not allowed in token.
+    // ':' is used to separate host and port.
+    if (node_by[0] == '_') {
+      res += "by=";
+      res += node_by;
+      res += ";";
+    } else {
+      res += "by=\"";
+      res += node_by;
+      res += "\";";
+    }
+  }
+  if ((params & FORWARDED_FOR) && !node_for.empty()) {
+    // We only quote IPv6 literal address only, which starts with '['.
+    if (node_for[0] == '[') {
+      res += "for=\"";
+      res += node_for;
+      res += "\";";
+    } else {
+      res += "for=";
+      res += node_for;
+      res += ";";
+    }
+  }
+  if ((params & FORWARDED_HOST) && !host.empty()) {
+    // Just be quoted to skip checking characters.
+    res += "host=\"";
+    res += host;
+    res += "\";";
+  }
+  if ((params & FORWARDED_PROTO) && !proto.empty()) {
+    // Scheme production rule only allow characters which are all in
+    // token.
+    res += "proto=";
+    res += proto;
+    res += ";";
+  }
+
+  if (res.empty()) {
+    return res;
+  }
+
+  res.erase(res.size() - 1);
+
+  return res;
+}
+
 std::string colorizeHeaders(const char *hdrs) {
   std::string nhdrs;
   const char *p = strchr(hdrs, '\n');
