@@ -67,20 +67,16 @@ Http2DownstreamConnection::~Http2DownstreamConnection() {
       error_code = NGHTTP2_INTERNAL_ERROR;
     }
 
-    if (downstream_->get_downstream_stream_id() != -1) {
-      if (LOG_ENABLED(INFO)) {
-        DCLOG(INFO, this) << "Submit RST_STREAM for DOWNSTREAM:" << downstream_
-                          << ", stream_id="
-                          << downstream_->get_downstream_stream_id()
-                          << ", error_code=" << error_code;
-      }
-
+    if (http2session_->get_state() == Http2Session::CONNECTED &&
+        downstream_->get_downstream_stream_id() != -1) {
       submit_rst_stream(downstream_, error_code);
 
-      http2session_->consume(downstream_->get_downstream_stream_id(),
-                             downstream_->get_response_datalen());
+      auto &resp = downstream_->response();
 
-      downstream_->reset_response_datalen();
+      http2session_->consume(downstream_->get_downstream_stream_id(),
+                             resp.unconsumed_body_length);
+
+      resp.unconsumed_body_length = 0;
 
       http2session_->signal_write();
     }
@@ -111,15 +107,18 @@ void Http2DownstreamConnection::detach_downstream(Downstream *downstream) {
   if (LOG_ENABLED(INFO)) {
     DCLOG(INFO, this) << "Detaching from DOWNSTREAM:" << downstream;
   }
+
+  auto &resp = downstream_->response();
+
   if (submit_rst_stream(downstream) == 0) {
     http2session_->signal_write();
   }
 
   if (downstream_->get_downstream_stream_id() != -1) {
     http2session_->consume(downstream_->get_downstream_stream_id(),
-                           downstream_->get_response_datalen());
+                           resp.unconsumed_body_length);
 
-    downstream_->reset_response_datalen();
+    resp.unconsumed_body_length = 0;
 
     http2session_->signal_write();
   }
@@ -143,7 +142,8 @@ int Http2DownstreamConnection::submit_rst_stream(Downstream *downstream,
       if (LOG_ENABLED(INFO)) {
         DCLOG(INFO, this) << "Submit RST_STREAM for DOWNSTREAM:" << downstream
                           << ", stream_id="
-                          << downstream->get_downstream_stream_id();
+                          << downstream->get_downstream_stream_id()
+                          << ", error_code=" << error_code;
       }
       rv = http2session_->submit_rst_stream(
           downstream->get_downstream_stream_id(), error_code);
@@ -170,6 +170,7 @@ ssize_t http2_data_read_callback(nghttp2_session *session, int32_t stream_id,
     // on the priority, DATA frame may come first.
     return NGHTTP2_ERR_DEFERRED;
   }
+  const auto &req = downstream->request();
   auto input = downstream->get_request_buf();
   auto nread = input->remove(buf, length);
   auto input_empty = input->rleft() == 0;
@@ -195,16 +196,18 @@ ssize_t http2_data_read_callback(nghttp2_session *session, int32_t stream_id,
       // If connection is upgraded, don't set EOF flag, since HTTP/1
       // will set MSG_COMPLETE to request state after upgrade response
       // header is seen.
-      (!downstream->get_upgrade_request() ||
+      (!req.upgrade_request ||
        (downstream->get_response_state() == Downstream::HEADER_COMPLETE &&
         !downstream->get_upgraded()))) {
 
     *data_flags |= NGHTTP2_DATA_FLAG_EOF;
 
-    auto &trailers = downstream->get_request_trailers();
+    const auto &trailers = req.fs.trailers();
     if (!trailers.empty()) {
       std::vector<nghttp2_nv> nva;
       nva.reserve(trailers.size());
+      // We cannot use nocopy version, since nva may be touched after
+      // Downstream object is deleted.
       http2::copy_headers_to_nva(nva, trailers);
       if (!nva.empty()) {
         rv = nghttp2_submit_trailer(session, stream_id, nva.data(), nva.size());
@@ -251,124 +254,166 @@ int Http2DownstreamConnection::push_request_headers() {
 
   downstream_->set_request_pending(false);
 
-  auto method = downstream_->get_request_method();
-  auto no_host_rewrite = get_config()->no_host_rewrite ||
-                         get_config()->http2_proxy ||
-                         get_config()->client_proxy || method == HTTP_CONNECT;
+  const auto &req = downstream_->request();
+
+  auto &httpconf = get_config()->http;
+  auto &http2conf = get_config()->http2;
+
+  auto no_host_rewrite =
+      httpconf.no_host_rewrite || get_config()->http2_proxy ||
+      get_config()->client_proxy || req.method == HTTP_CONNECT;
 
   // http2session_ has already in CONNECTED state, so we can get
   // addr_idx here.
   auto addr_idx = http2session_->get_addr_idx();
   auto group = http2session_->get_group();
-  auto downstream_hostport = get_config()
-                                 ->downstream_addr_groups[group]
-                                 .addrs[addr_idx]
-                                 .hostport.get();
+  const auto &downstream_hostport =
+      get_config()->conn.downstream.addr_groups[group].addrs[addr_idx].hostport;
 
   // For HTTP/1.0 request, there is no authority in request.  In that
   // case, we use backend server's host nonetheless.
-  const char *authority = downstream_hostport;
-  auto &req_authority = downstream_->get_request_http2_authority();
-  if (no_host_rewrite && !req_authority.empty()) {
-    authority = req_authority.c_str();
+  auto authority = StringRef(downstream_hostport);
+
+  if (no_host_rewrite && !req.authority.empty()) {
+    authority = StringRef(req.authority);
   }
 
-  if (!authority) {
-    authority = downstream_hostport;
+  downstream_->set_request_downstream_host(authority.str());
+
+  size_t num_cookies = 0;
+  if (!http2conf.no_cookie_crumbling) {
+    num_cookies = downstream_->count_crumble_request_cookie();
   }
 
-  downstream_->set_request_downstream_host(authority);
-
-  auto nheader = downstream_->get_request_headers().size();
-
-  Headers cookies;
-  if (!get_config()->http2_no_cookie_crumbling) {
-    cookies = downstream_->crumble_request_cookie();
-  }
-
-  // 8 means:
+  // 9 means:
   // 1. :method
   // 2. :scheme
   // 3. :path
-  // 4. :authority
+  // 4. :authority (or host)
   // 5. via (optional)
   // 6. x-forwarded-for (optional)
   // 7. x-forwarded-proto (optional)
   // 8. te (optional)
+  // 9. forwarded (optional)
   auto nva = std::vector<nghttp2_nv>();
-  nva.reserve(nheader + 8 + cookies.size() +
-              get_config()->add_request_headers.size());
+  nva.reserve(req.fs.headers().size() + 9 + num_cookies +
+              httpconf.add_request_headers.size());
 
-  nva.push_back(http2::make_nv_lc(":method", http2::to_method_string(method)));
+  nva.push_back(
+      http2::make_nv_lc_nocopy(":method", http2::to_method_string(req.method)));
 
-  auto &scheme = downstream_->get_request_http2_scheme();
+  if (req.method != HTTP_CONNECT) {
+    assert(!req.scheme.empty());
 
-  nva.push_back(http2::make_nv_lc(":authority", authority));
+    nva.push_back(http2::make_nv_ls_nocopy(":scheme", req.scheme));
 
-  if (method != HTTP_CONNECT) {
-    assert(!scheme.empty());
-
-    nva.push_back(http2::make_nv_ls(":scheme", scheme));
-
-    auto &path = downstream_->get_request_path();
-    if (method == HTTP_OPTIONS && path.empty()) {
+    if (req.method == HTTP_OPTIONS && req.path.empty()) {
       nva.push_back(http2::make_nv_ll(":path", "*"));
     } else {
-      nva.push_back(http2::make_nv_ls(":path", path));
+      nva.push_back(http2::make_nv_ls_nocopy(":path", req.path));
     }
+
+    if (!req.no_authority) {
+      nva.push_back(http2::make_nv_ls_nocopy(":authority", authority));
+    } else {
+      nva.push_back(http2::make_nv_ls_nocopy("host", authority));
+    }
+  } else {
+    nva.push_back(http2::make_nv_ls_nocopy(":authority", authority));
   }
 
-  http2::copy_headers_to_nva(nva, downstream_->get_request_headers());
+  http2::copy_headers_to_nva_nocopy(nva, req.fs.headers());
 
   bool chunked_encoding = false;
-  auto transfer_encoding =
-      downstream_->get_request_header(http2::HD_TRANSFER_ENCODING);
+  auto transfer_encoding = req.fs.header(http2::HD_TRANSFER_ENCODING);
   if (transfer_encoding &&
       util::strieq_l("chunked", (*transfer_encoding).value)) {
     chunked_encoding = true;
   }
 
-  for (auto &nv : cookies) {
-    nva.push_back(http2::make_nv(nv.name, nv.value, nv.no_index));
+  if (!http2conf.no_cookie_crumbling) {
+    downstream_->crumble_request_cookie(nva);
   }
 
+  auto upstream = downstream_->get_upstream();
+  auto handler = upstream->get_client_handler();
+
+  std::string forwarded_value;
+
+  auto &fwdconf = httpconf.forwarded;
+
+  auto fwd =
+      fwdconf.strip_incoming ? nullptr : req.fs.header(http2::HD_FORWARDED);
+
+  if (fwdconf.params) {
+    auto params = fwdconf.params;
+
+    if (get_config()->http2_proxy || get_config()->client_proxy ||
+        req.method == HTTP_CONNECT) {
+      params &= ~FORWARDED_PROTO;
+    }
+
+    auto value = http::create_forwarded(params, handler->get_forwarded_by(),
+                                        handler->get_forwarded_for(),
+                                        req.authority, req.scheme);
+    if (fwd || !value.empty()) {
+      if (fwd) {
+        forwarded_value = fwd->value;
+
+        if (!value.empty()) {
+          forwarded_value += ", ";
+        }
+      }
+
+      forwarded_value += value;
+
+      nva.push_back(http2::make_nv_ls("forwarded", forwarded_value));
+    }
+  } else if (fwd) {
+    nva.push_back(http2::make_nv_ls_nocopy("forwarded", fwd->value));
+    forwarded_value = fwd->value;
+  }
+
+  auto &xffconf = httpconf.xff;
+
+  auto xff = xffconf.strip_incoming ? nullptr
+                                    : req.fs.header(http2::HD_X_FORWARDED_FOR);
+
   std::string xff_value;
-  auto xff = downstream_->get_request_header(http2::HD_X_FORWARDED_FOR);
-  if (get_config()->add_x_forwarded_for) {
-    if (xff && !get_config()->strip_incoming_x_forwarded_for) {
+
+  if (xffconf.add) {
+    if (xff) {
       xff_value = (*xff).value;
       xff_value += ", ";
     }
-    xff_value +=
-        downstream_->get_upstream()->get_client_handler()->get_ipaddr();
+    xff_value += upstream->get_client_handler()->get_ipaddr();
     nva.push_back(http2::make_nv_ls("x-forwarded-for", xff_value));
-  } else if (xff && !get_config()->strip_incoming_x_forwarded_for) {
-    nva.push_back(http2::make_nv_ls("x-forwarded-for", (*xff).value));
+  } else if (xff) {
+    nva.push_back(http2::make_nv_ls_nocopy("x-forwarded-for", (*xff).value));
   }
 
   if (!get_config()->http2_proxy && !get_config()->client_proxy &&
-      downstream_->get_request_method() != HTTP_CONNECT) {
+      req.method != HTTP_CONNECT) {
     // We use same protocol with :scheme header field
-    nva.push_back(http2::make_nv_ls("x-forwarded-proto", scheme));
+    nva.push_back(http2::make_nv_ls_nocopy("x-forwarded-proto", req.scheme));
   }
 
   std::string via_value;
-  auto via = downstream_->get_request_header(http2::HD_VIA);
-  if (get_config()->no_via) {
+  auto via = req.fs.header(http2::HD_VIA);
+  if (httpconf.no_via) {
     if (via) {
-      nva.push_back(http2::make_nv_ls("via", (*via).value));
+      nva.push_back(http2::make_nv_ls_nocopy("via", (*via).value));
     }
   } else {
     if (via) {
       via_value = (*via).value;
       via_value += ", ";
     }
-    via_value += http::create_via_header_value(
-        downstream_->get_request_major(), downstream_->get_request_minor());
+    via_value += http::create_via_header_value(req.http_major, req.http_minor);
     nva.push_back(http2::make_nv_ls("via", via_value));
   }
 
-  auto te = downstream_->get_request_header(http2::HD_TE);
+  auto te = req.fs.header(http2::HD_TE);
   // HTTP/1 upstream request can contain keyword other than
   // "trailers".  We just forward "trailers".
   // TODO more strict handling required here.
@@ -376,8 +421,8 @@ int Http2DownstreamConnection::push_request_headers() {
     nva.push_back(http2::make_nv_ll("te", "trailers"));
   }
 
-  for (auto &p : get_config()->add_request_headers) {
-    nva.push_back(http2::make_nv(p.first, p.second));
+  for (auto &p : httpconf.add_request_headers) {
+    nva.push_back(http2::make_nv_nocopy(p.first, p.second));
   }
 
   if (LOG_ENABLED(INFO)) {
@@ -388,21 +433,18 @@ int Http2DownstreamConnection::push_request_headers() {
     DCLOG(INFO, this) << "HTTP request headers\n" << ss.str();
   }
 
-  auto content_length =
-      downstream_->get_request_header(http2::HD_CONTENT_LENGTH);
+  auto content_length = req.fs.header(http2::HD_CONTENT_LENGTH);
   // TODO check content-length: 0 case
 
-  if (downstream_->get_request_method() == HTTP_CONNECT || chunked_encoding ||
-      content_length || downstream_->get_request_http2_expect_body()) {
+  if (req.method == HTTP_CONNECT || chunked_encoding || content_length ||
+      req.http2_expect_body) {
     // Request-body is expected.
     nghttp2_data_provider data_prd;
     data_prd.source.ptr = this;
     data_prd.read_callback = http2_data_read_callback;
-    rv = http2session_->submit_request(this, downstream_->get_priority(),
-                                       nva.data(), nva.size(), &data_prd);
+    rv = http2session_->submit_request(this, nva.data(), nva.size(), &data_prd);
   } else {
-    rv = http2session_->submit_request(this, downstream_->get_priority(),
-                                       nva.data(), nva.size(), nullptr);
+    rv = http2session_->submit_request(this, nva.data(), nva.size(), nullptr);
   }
   if (rv != 0) {
     DCLOG(FATAL, this) << "nghttp2_submit_request() failed";
@@ -462,8 +504,6 @@ int Http2DownstreamConnection::resume_read(IOCtrlReason reason,
   }
 
   if (consumed > 0) {
-    assert(downstream_->get_response_datalen() >= consumed);
-
     rv = http2session_->consume(downstream_->get_downstream_stream_id(),
                                 consumed);
 
@@ -471,7 +511,9 @@ int Http2DownstreamConnection::resume_read(IOCtrlReason reason,
       return -1;
     }
 
-    downstream_->dec_response_datalen(consumed);
+    auto &resp = downstream_->response();
+
+    resp.unconsumed_body_length -= consumed;
 
     http2session_->signal_write();
   }
@@ -504,24 +546,6 @@ StreamData *Http2DownstreamConnection::detach_stream_data() {
   return nullptr;
 }
 
-int Http2DownstreamConnection::on_priority_change(int32_t pri) {
-  int rv;
-  if (downstream_->get_priority() == pri) {
-    return 0;
-  }
-  downstream_->set_priority(pri);
-  if (http2session_->get_state() != Http2Session::CONNECTED) {
-    return 0;
-  }
-  rv = http2session_->submit_priority(this, pri);
-  if (rv != 0) {
-    DLOG(FATAL, this) << "nghttp2_submit_priority() failed";
-    return -1;
-  }
-  http2session_->signal_write();
-  return 0;
-}
-
 int Http2DownstreamConnection::on_timeout() {
   if (!downstream_) {
     return 0;
