@@ -146,10 +146,10 @@ Http2Session::Http2Session(struct ev_loop *loop, SSL_CTX *ssl_ctx,
                            ConnectBlocker *connect_blocker, Worker *worker,
                            size_t group, size_t idx)
     : conn_(loop, -1, nullptr, worker->get_mcpool(),
-            get_config()->downstream_write_timeout,
-            get_config()->downstream_read_timeout, 0, 0, 0, 0, writecb, readcb,
-            timeoutcb, this, get_config()->tls_dyn_rec_warmup_threshold,
-            get_config()->tls_dyn_rec_idle_timeout),
+            get_config()->conn.downstream.timeout.write,
+            get_config()->conn.downstream.timeout.read, {}, {}, writecb, readcb,
+            timeoutcb, this, get_config()->tls.dyn_rec.warmup_threshold,
+            get_config()->tls.dyn_rec.idle_timeout),
       worker_(worker), connect_blocker_(connect_blocker), ssl_ctx_(ssl_ctx),
       session_(nullptr), data_pending_(nullptr), data_pendinglen_(0),
       addr_idx_(0), group_(group), index_(idx), state_(DISCONNECTED),
@@ -240,13 +240,13 @@ int Http2Session::disconnect(bool hard) {
 int Http2Session::check_cert() {
   return ssl::check_cert(
       conn_.tls.ssl,
-      &get_config()->downstream_addr_groups[group_].addrs[addr_idx_]);
+      &get_config()->conn.downstream.addr_groups[group_].addrs[addr_idx_]);
 }
 
 int Http2Session::initiate_connection() {
   int rv = 0;
 
-  auto &addrs = get_config()->downstream_addr_groups[group_].addrs;
+  auto &addrs = get_config()->conn.downstream.addr_groups[group_].addrs;
 
   if (state_ == DISCONNECTED) {
     if (connect_blocker_->blocked()) {
@@ -271,27 +271,24 @@ int Http2Session::initiate_connection() {
 
   auto &downstream_addr = addrs[addr_idx_];
 
-  if (get_config()->downstream_http_proxy_host && state_ == DISCONNECTED) {
+  const auto &proxy = get_config()->downstream_http_proxy;
+  if (!proxy.host.empty() && state_ == DISCONNECTED) {
     if (LOG_ENABLED(INFO)) {
-      SSLOG(INFO, this) << "Connecting to the proxy "
-                        << get_config()->downstream_http_proxy_host.get() << ":"
-                        << get_config()->downstream_http_proxy_port;
+      SSLOG(INFO, this) << "Connecting to the proxy " << proxy.host << ":"
+                        << proxy.port;
     }
 
-    conn_.fd = util::create_nonblock_socket(
-        get_config()->downstream_http_proxy_addr.su.storage.ss_family);
+    conn_.fd = util::create_nonblock_socket(proxy.addr.su.storage.ss_family);
 
     if (conn_.fd == -1) {
       connect_blocker_->on_failure();
       return -1;
     }
 
-    rv = connect(conn_.fd, &get_config()->downstream_http_proxy_addr.su.sa,
-                 get_config()->downstream_http_proxy_addr.len);
+    rv = connect(conn_.fd, &proxy.addr.su.sa, proxy.addr.len);
     if (rv != 0 && errno != EINPROGRESS) {
-      SSLOG(ERROR, this) << "Failed to connect to the proxy "
-                         << get_config()->downstream_http_proxy_host.get()
-                         << ":" << get_config()->downstream_http_proxy_port;
+      SSLOG(ERROR, this) << "Failed to connect to the proxy " << proxy.host
+                         << ":" << proxy.port;
       connect_blocker_->on_failure();
       return -1;
     }
@@ -334,18 +331,15 @@ int Http2Session::initiate_connection() {
         conn_.set_ssl(ssl);
       }
 
-      const char *sni_name = nullptr;
-      if (get_config()->backend_tls_sni_name) {
-        sni_name = get_config()->backend_tls_sni_name.get();
-      } else {
-        sni_name = downstream_addr.host.get();
-      }
+      auto sni_name = !get_config()->tls.backend_sni_name.empty()
+                          ? StringRef(get_config()->tls.backend_sni_name)
+                          : StringRef(downstream_addr.host);
 
-      if (sni_name && !util::numeric_host(sni_name)) {
+      if (!util::numeric_host(sni_name.c_str())) {
         // TLS extensions: SNI. There is no documentation about the return
         // code for this function (actually this is macro wrapping SSL_ctrl
         // at the time of this writing).
-        SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name);
+        SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name.c_str());
       }
       // If state_ == PROXY_CONNECTED, we has connected to the proxy
       // using conn_.fd and tunnel has been established.
@@ -515,23 +509,21 @@ int Http2Session::downstream_connect_proxy() {
     SSLOG(INFO, this) << "Connected to the proxy";
   }
   auto &downstream_addr =
-      get_config()->downstream_addr_groups[group_].addrs[addr_idx_];
+      get_config()->conn.downstream.addr_groups[group_].addrs[addr_idx_];
 
   std::string req = "CONNECT ";
-  req += downstream_addr.hostport.get();
+  req.append(downstream_addr.hostport.c_str(), downstream_addr.hostport.size());
   if (downstream_addr.port == 80 || downstream_addr.port == 443) {
-    req += ":";
+    req += ':';
     req += util::utos(downstream_addr.port);
   }
   req += " HTTP/1.1\r\nHost: ";
-  req += downstream_addr.host.get();
+  req.append(downstream_addr.host.c_str(), downstream_addr.host.size());
   req += "\r\n";
-  if (get_config()->downstream_http_proxy_userinfo) {
+  const auto &proxy = get_config()->downstream_http_proxy;
+  if (!proxy.userinfo.empty()) {
     req += "Proxy-Authorization: Basic ";
-    size_t len = strlen(get_config()->downstream_http_proxy_userinfo.get());
-    req += base64::encode(get_config()->downstream_http_proxy_userinfo.get(),
-                          get_config()->downstream_http_proxy_userinfo.get() +
-                              len);
+    req += base64::encode(std::begin(proxy.userinfo), std::end(proxy.userinfo));
     req += "\r\n";
   }
   req += "\r\n";
@@ -567,7 +559,7 @@ void Http2Session::remove_stream_data(StreamData *sd) {
   delete sd;
 }
 
-int Http2Session::submit_request(Http2DownstreamConnection *dconn, int32_t pri,
+int Http2Session::submit_request(Http2DownstreamConnection *dconn,
                                  const nghttp2_nv *nva, size_t nvlen,
                                  const nghttp2_data_provider *data_prd) {
   assert(state_ == CONNECTED);
@@ -605,30 +597,6 @@ int Http2Session::submit_rst_stream(int32_t stream_id, uint32_t error_code) {
   return 0;
 }
 
-int Http2Session::submit_priority(Http2DownstreamConnection *dconn,
-                                  int32_t pri) {
-  assert(state_ == CONNECTED);
-  if (!dconn) {
-    return 0;
-  }
-  int rv;
-
-  // TODO Disabled temporarily
-
-  // rv = nghttp2_submit_priority(session_, NGHTTP2_FLAG_NONE,
-  //                              dconn->get_downstream()->
-  //                              get_downstream_stream_id(), pri);
-
-  rv = 0;
-
-  if (rv < NGHTTP2_ERR_FATAL) {
-    SSLOG(FATAL, this) << "nghttp2_submit_priority() failed: "
-                       << nghttp2_strerror(rv);
-    return -1;
-  }
-  return 0;
-}
-
 nghttp2_session *Http2Session::get_session() const { return session_; }
 
 bool Http2Session::get_flow_control() const { return flow_control_; }
@@ -752,42 +720,24 @@ int on_header_callback(nghttp2_session *session, const nghttp2_frame *frame,
     return 0;
   }
 
+  auto &resp = downstream->response();
+
   switch (frame->hd.type) {
   case NGHTTP2_HEADERS: {
     auto trailer = frame->headers.cat == NGHTTP2_HCAT_HEADERS &&
                    !downstream->get_expect_final_response();
 
-    if (downstream->get_response_headers_sum() + namelen + valuelen >
-            get_config()->header_field_buffer ||
-        downstream->get_response_headers().size() >=
-            get_config()->max_header_fields) {
-      if (LOG_ENABLED(INFO)) {
-        DLOG(INFO, downstream)
-            << "Too large or many header field size="
-            << downstream->get_response_headers_sum() + namelen + valuelen
-            << ", num=" << downstream->get_response_headers().size() + 1;
-      }
-
-      if (trailer) {
-        // we don't care trailer part exceeds header size limit; just
-        // discard it.
-        return 0;
-      }
-
-      return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-    }
-
     if (trailer) {
       // just store header fields for trailer part
-      downstream->add_response_trailer(name, namelen, value, valuelen,
-                                       flags & NGHTTP2_NV_FLAG_NO_INDEX, -1);
+      resp.fs.add_trailer(name, namelen, value, valuelen,
+                          flags & NGHTTP2_NV_FLAG_NO_INDEX, -1);
       return 0;
     }
 
     auto token = http2::lookup_token(name, namelen);
 
-    downstream->add_response_header(name, namelen, value, valuelen,
-                                    flags & NGHTTP2_NV_FLAG_NO_INDEX, token);
+    resp.fs.add_header(name, namelen, value, valuelen,
+                       flags & NGHTTP2_NV_FLAG_NO_INDEX, token);
     return 0;
   }
   case NGHTTP2_PUSH_PROMISE: {
@@ -803,25 +753,11 @@ int on_header_callback(nghttp2_session *session, const nghttp2_frame *frame,
 
     assert(promised_downstream);
 
-    if (promised_downstream->get_request_headers_sum() + namelen + valuelen >
-            get_config()->header_field_buffer ||
-        promised_downstream->get_request_headers().size() >=
-            get_config()->max_header_fields) {
-      if (LOG_ENABLED(INFO)) {
-        DLOG(INFO, promised_downstream)
-            << "Too large or many header field size="
-            << promised_downstream->get_request_headers_sum() + namelen +
-                   valuelen << ", num="
-            << promised_downstream->get_request_headers().size() + 1;
-      }
-
-      return NGHTTP2_ERR_TEMPORAL_CALLBACK_FAILURE;
-    }
+    auto &promised_req = promised_downstream->request();
 
     auto token = http2::lookup_token(name, namelen);
-    promised_downstream->add_request_header(name, namelen, value, valuelen,
-                                            flags & NGHTTP2_NV_FLAG_NO_INDEX,
-                                            token);
+    promised_req.fs.add_header(name, namelen, value, valuelen,
+                               flags & NGHTTP2_NV_FLAG_NO_INDEX, token);
     return 0;
   }
   }
@@ -890,18 +826,20 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
   int rv;
 
   auto upstream = downstream->get_upstream();
+  const auto &req = downstream->request();
+  auto &resp = downstream->response();
 
-  auto &nva = downstream->get_response_headers();
+  auto &nva = resp.fs.headers();
 
   downstream->set_expect_final_response(false);
 
-  auto status = downstream->get_response_header(http2::HD__STATUS);
+  auto status = resp.fs.header(http2::HD__STATUS);
   // libnghttp2 guarantees this exists and can be parsed
   auto status_code = http2::parse_http_status_code(status->value);
 
-  downstream->set_response_http_status(status_code);
-  downstream->set_response_major(2);
-  downstream->set_response_minor(0);
+  resp.http_status = status_code;
+  resp.http_major = 2;
+  resp.http_minor = 0;
 
   if (LOG_ENABLED(INFO)) {
     std::stringstream ss;
@@ -937,7 +875,7 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
   downstream->check_upgrade_fulfilled();
 
   if (downstream->get_upgraded()) {
-    downstream->set_response_connection_close(true);
+    resp.connection_close = true;
     // On upgrade sucess, both ends can send data
     if (upstream->resume_read(SHRPX_NO_BUFFER, downstream, 0) != 0) {
       // If resume_read fails, just drop connection. Not ideal.
@@ -950,29 +888,24 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
           << "HTTP upgrade success. stream_id=" << frame->hd.stream_id;
     }
   } else {
-    auto content_length =
-        downstream->get_response_header(http2::HD_CONTENT_LENGTH);
+    auto content_length = resp.fs.header(http2::HD_CONTENT_LENGTH);
     if (content_length) {
       // libnghttp2 guarantees this can be parsed
-      auto len = util::parse_uint(content_length->value);
-      downstream->set_response_content_length(len);
+      resp.fs.content_length = util::parse_uint(content_length->value);
     }
 
-    if (downstream->get_response_content_length() == -1 &&
-        downstream->expect_response_body()) {
+    if (resp.fs.content_length == -1 && downstream->expect_response_body()) {
       // Here we have response body but Content-Length is not known in
       // advance.
-      if (downstream->get_request_major() <= 0 ||
-          (downstream->get_request_major() == 1 &&
-           downstream->get_request_minor() == 0)) {
+      if (req.http_major <= 0 || (req.http_major == 1 && req.http_minor == 0)) {
         // We simply close connection for pre-HTTP/1.1 in this case.
-        downstream->set_response_connection_close(true);
+        resp.connection_close = true;
       } else {
         // Otherwise, use chunked encoding to keep upstream connection
         // open.  In HTTP2, we are supporsed not to receive
         // transfer-encoding.
-        downstream->add_response_header("transfer-encoding", "chunked",
-                                        http2::HD_TRANSFER_ENCODING);
+        resp.fs.add_header("transfer-encoding", "chunked",
+                           http2::HD_TRANSFER_ENCODING);
         downstream->set_chunked_response(true);
       }
     }
@@ -1212,7 +1145,10 @@ int on_data_chunk_recv_callback(nghttp2_session *session, uint8_t flags,
 
   downstream->reset_downstream_rtimer();
 
-  downstream->add_response_bodylen(len);
+  auto &resp = downstream->response();
+
+  resp.recv_body_length += len;
+  resp.unconsumed_body_length += len;
 
   auto upstream = downstream->get_upstream();
   rv = upstream->on_downstream_body(downstream, data, len, false);
@@ -1226,8 +1162,6 @@ int on_data_chunk_recv_callback(nghttp2_session *session, uint8_t flags,
     downstream->set_response_state(Downstream::MSG_RESET);
   }
 
-  downstream->add_response_datalen(len);
-
   call_downstream_readcb(http2session, downstream);
   return 0;
 }
@@ -1375,9 +1309,10 @@ int Http2Session::connection_made() {
     }
   }
 
-  rv = nghttp2_session_client_new2(&session_,
-                                   get_config()->http2_downstream_callbacks,
-                                   this, get_config()->http2_client_option);
+  auto &http2conf = get_config()->http2;
+
+  rv = nghttp2_session_client_new2(&session_, http2conf.downstream.callbacks,
+                                   this, http2conf.downstream.option);
 
   if (rv != 0) {
     return -1;
@@ -1388,12 +1323,12 @@ int Http2Session::connection_made() {
   std::array<nghttp2_settings_entry, 3> entry;
   size_t nentry = 2;
   entry[0].settings_id = NGHTTP2_SETTINGS_MAX_CONCURRENT_STREAMS;
-  entry[0].value = get_config()->http2_max_concurrent_streams;
+  entry[0].value = http2conf.max_concurrent_streams;
 
   entry[1].settings_id = NGHTTP2_SETTINGS_INITIAL_WINDOW_SIZE;
-  entry[1].value = (1 << get_config()->http2_downstream_window_bits) - 1;
+  entry[1].value = (1 << http2conf.downstream.window_bits) - 1;
 
-  if (get_config()->no_server_push || get_config()->http2_proxy ||
+  if (http2conf.no_server_push || get_config()->http2_proxy ||
       get_config()->client_proxy) {
     entry[nentry].settings_id = NGHTTP2_SETTINGS_ENABLE_PUSH;
     entry[nentry].value = 0;
@@ -1406,17 +1341,17 @@ int Http2Session::connection_made() {
     return -1;
   }
 
-  if (get_config()->http2_downstream_connection_window_bits > 16) {
-    int32_t delta =
-        (1 << get_config()->http2_downstream_connection_window_bits) - 1 -
-        NGHTTP2_INITIAL_CONNECTION_WINDOW_SIZE;
+  auto connection_window_bits = http2conf.downstream.connection_window_bits;
+  if (connection_window_bits > 16) {
+    int32_t delta = (1 << connection_window_bits) - 1 -
+                    NGHTTP2_INITIAL_CONNECTION_WINDOW_SIZE;
     rv = nghttp2_submit_window_update(session_, NGHTTP2_FLAG_NONE, 0, delta);
     if (rv != 0) {
       return -1;
     }
   }
 
-  auto must_terminate = !get_config()->downstream_no_tls &&
+  auto must_terminate = !get_config()->conn.downstream.no_tls &&
                         !nghttp2::ssl::check_http2_requirement(conn_.tls.ssl);
 
   if (must_terminate) {
@@ -1784,7 +1719,7 @@ int Http2Session::tls_handshake() {
     SSLOG(INFO, this) << "SSL/TLS handshake completed";
   }
 
-  if (!get_config()->downstream_no_tls && !get_config()->insecure &&
+  if (!get_config()->conn.downstream.no_tls && !get_config()->tls.insecure &&
       check_cert() != 0) {
     return -1;
   }
@@ -1927,14 +1862,15 @@ int Http2Session::handle_downstream_push_promise(Downstream *downstream,
 
 int Http2Session::handle_downstream_push_promise_complete(
     Downstream *downstream, Downstream *promised_downstream) {
-  auto authority =
-      promised_downstream->get_request_header(http2::HD__AUTHORITY);
-  auto path = promised_downstream->get_request_header(http2::HD__PATH);
-  auto method = promised_downstream->get_request_header(http2::HD__METHOD);
-  auto scheme = promised_downstream->get_request_header(http2::HD__SCHEME);
+  auto &promised_req = promised_downstream->request();
+
+  auto authority = promised_req.fs.header(http2::HD__AUTHORITY);
+  auto path = promised_req.fs.header(http2::HD__PATH);
+  auto method = promised_req.fs.header(http2::HD__METHOD);
+  auto scheme = promised_req.fs.header(http2::HD__SCHEME);
 
   if (!authority) {
-    authority = promised_downstream->get_request_header(http2::HD_HOST);
+    authority = promised_req.fs.header(http2::HD_HOST);
   }
 
   auto method_token = http2::lookup_method_token(method->value);
@@ -1949,17 +1885,16 @@ int Http2Session::handle_downstream_push_promise_complete(
   // TODO Rewrite authority if we enabled rewrite host.  But we
   // really don't know how to rewrite host.  Should we use the same
   // host in associated stream?
-  promised_downstream->set_request_http2_authority(
-      http2::value_to_str(authority));
-  promised_downstream->set_request_method(method_token);
+  promised_req.authority = http2::value_to_str(authority);
+  promised_req.method = method_token;
   // libnghttp2 ensures that we don't have CONNECT method in
   // PUSH_PROMISE, and guarantees that :scheme exists.
-  promised_downstream->set_request_http2_scheme(http2::value_to_str(scheme));
+  promised_req.scheme = http2::value_to_str(scheme);
 
   // For server-wide OPTIONS request, path is empty.
   if (method_token != HTTP_OPTIONS || path->value != "*") {
-    promised_downstream->set_request_path(http2::rewrite_clean_path(
-        std::begin(path->value), std::end(path->value)));
+    promised_req.path = http2::rewrite_clean_path(std::begin(path->value),
+                                                  std::end(path->value));
   }
 
   promised_downstream->inspect_http2_request();
