@@ -56,7 +56,7 @@ public:
   virtual int downstream_write(DownstreamConnection *dconn);
   virtual int downstream_eof(DownstreamConnection *dconn);
   virtual int downstream_error(DownstreamConnection *dconn, int events);
-  Downstream *add_pending_downstream(int32_t stream_id, int32_t priority);
+  Downstream *add_pending_downstream(int32_t stream_id);
   void remove_downstream(Downstream *downstream);
 
   int rst_stream(Downstream *downstream, int status_code);
@@ -82,6 +82,14 @@ public:
   virtual void response_drain(size_t n);
   virtual bool response_empty() const;
 
+  virtual Downstream *on_downstream_push_promise(Downstream *downstream,
+                                                 int32_t promised_stream_id);
+  virtual int
+  on_downstream_push_promise_complete(Downstream *downstream,
+                                      Downstream *promised_downstream);
+  virtual bool push_enabled() const;
+  virtual void cancel_premature_downstream(Downstream *promised_downstream);
+
   bool get_flow_control() const;
 
   int consume(int32_t stream_id, size_t len);
