@@ -28,33 +28,60 @@
 #include "shrpx.h"
 
 #include <vector>
+#include <mutex>
 
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 
-#include <event.h>
+#include <ev.h>
+
+#ifdef HAVE_NEVERBLEED
+#include <neverbleed.h>
+#endif // HAVE_NEVERBLEED
 
 namespace shrpx {
 
 class ClientHandler;
-struct WorkerStat;
+class Worker;
 class DownstreamConnectionPool;
+struct DownstreamAddr;
 
 namespace ssl {
 
-SSL_CTX *create_ssl_context(const char *private_key_file,
-                            const char *cert_file);
+// This struct stores the additional information per SSL_CTX.  This is
+// attached to SSL_CTX using SSL_CTX_set_app_data().
+struct TLSContextData {
+  // Protects ocsp_data;
+  std::mutex mu;
+  // OCSP response
+  std::shared_ptr<std::vector<uint8_t>> ocsp_data;
 
-SSL_CTX *create_ssl_client_context();
+  // Path to certificate file
+  const char *cert_file;
+};
 
-ClientHandler *accept_connection(event_base *evbase,
-                                 bufferevent_rate_limit_group *rate_limit_group,
-                                 SSL_CTX *ssl_ctx, evutil_socket_t fd,
-                                 sockaddr *addr, int addrlen,
-                                 WorkerStat *worker_stat,
-                                 DownstreamConnectionPool *dconn_pool);
+// Create server side SSL_CTX
+SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file
+#ifdef HAVE_NEVERBLEED
+                            ,
+                            neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+                            );
 
-int check_cert(SSL *ssl);
+// Create client side SSL_CTX
+SSL_CTX *create_ssl_client_context(
+#ifdef HAVE_NEVERBLEED
+    neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+    );
+
+ClientHandler *accept_connection(Worker *worker, int fd, sockaddr *addr,
+                                 int addrlen);
+
+// Check peer's certificate against first downstream address in
+// Config::downstream_addrs.  We only consider first downstream since
+// we use this function for HTTP/2 downstream link only.
+int check_cert(SSL *ssl, const DownstreamAddr *addr);
 
 // Retrieves DNS and IP address in subjectAltNames and commonName from
 // the |cert|.
@@ -81,41 +108,47 @@ void get_altnames(X509 *cert, std::vector<std::string> &dns_names,
 // them. If there is a match, its SSL_CTX is returned. If none
 // matches, query is continued to the next character.
 
-struct CertNode {
-  // SSL_CTX for exact match
+struct WildcardCert {
   SSL_CTX *ssl_ctx;
+  char *hostname;
+  size_t hostnamelen;
+};
+
+struct CertNode {
   // list of wildcard domain name and its SSL_CTX pair, the wildcard
   // '*' appears in this position.
-  std::vector<std::pair<char *, SSL_CTX *>> wildcard_certs;
+  std::vector<WildcardCert> wildcard_certs;
   // Next CertNode index of CertLookupTree::nodes
-  std::vector<CertNode *> next;
+  std::vector<std::unique_ptr<CertNode>> next;
+  // SSL_CTX for exact match
+  SSL_CTX *ssl_ctx;
   char *str;
   // [first, last) in the reverse direction in str, first >=
   // last. This indices only work for str member.
   int first, last;
 };
 
-struct CertLookupTree {
-  std::vector<SSL_CTX *> certs;
-  std::vector<char *> hosts;
-  CertNode *root;
+class CertLookupTree {
+public:
+  CertLookupTree();
+
+  // Adds |ssl_ctx| with hostname pattern |hostname| with length |len|
+  // to the lookup tree.  The |hostname| must be NULL-terminated.
+  void add_cert(SSL_CTX *ssl_ctx, const char *hostname, size_t len);
+
+  // Looks up SSL_CTX using the given |hostname| with length |len|.
+  // If more than one SSL_CTX which matches the query, it is undefined
+  // which one is returned.  The |hostname| must be NULL-terminated.
+  // If no matching SSL_CTX found, returns NULL.
+  SSL_CTX *lookup(const char *hostname, size_t len);
+
+private:
+  CertNode root_;
+  // Stores pointers to copied hostname when adding hostname and
+  // ssl_ctx pair.
+  std::vector<std::unique_ptr<char[]>> hosts_;
 };
 
-CertLookupTree *cert_lookup_tree_new();
-void cert_lookup_tree_del(CertLookupTree *lt);
-
-// Adds |ssl_ctx| with hostname pattern |hostname| with length |len|
-// to the lookup tree |lt|. The |hostname| must be NULL-terminated.
-void cert_lookup_tree_add_cert(CertLookupTree *lt, SSL_CTX *ssl_ctx,
-                               const char *hostname, size_t len);
-
-// Looks up SSL_CTX using the given |hostname| with length |len|. If
-// more than one SSL_CTX which matches the query, it is undefined
-// which one is returned. The |hostname| must be NULL-terminated. If
-// no matching SSL_CTX found, returns NULL.
-SSL_CTX *cert_lookup_tree_lookup(CertLookupTree *lt, const char *hostname,
-                                 size_t len);
-
 // Adds |ssl_ctx| to lookup tree |lt| using hostnames read from
 // |certfile|. The subjectAltNames and commonName are considered as
 // eligible hostname. This function returns 0 if it succeeds, or -1.
@@ -125,7 +158,7 @@ int cert_lookup_tree_add_cert_from_file(CertLookupTree *lt, SSL_CTX *ssl_ctx,
 
 // Returns true if |needle| which has |len| bytes is included in the
 // protocol list |protos|.
-bool in_proto_list(const std::vector<char *> &protos,
+bool in_proto_list(const std::vector<std::string> &protos,
                    const unsigned char *needle, size_t len);
 
 // Returns true if security requirement for HTTP/2 is fulfilled.
@@ -134,9 +167,49 @@ bool check_http2_requirement(SSL *ssl);
 // Returns SSL/TLS option mask to disable SSL/TLS protocol version not
 // included in |tls_proto_list|.  The returned mask can be directly
 // passed to SSL_CTX_set_options().
-long int create_tls_proto_mask(const std::vector<char *> &tls_proto_list);
+long int create_tls_proto_mask(const std::vector<std::string> &tls_proto_list);
 
-std::vector<unsigned char> set_alpn_prefs(const std::vector<char *> &protos);
+std::vector<unsigned char>
+set_alpn_prefs(const std::vector<std::string> &protos);
+
+// Setups server side SSL_CTX.  This function inspects get_config()
+// and if upstream_no_tls is true, returns nullptr.  Otherwise
+// construct default SSL_CTX.  If subcerts are available
+// (get_config()->subcerts), caller should provide CertLookupTree
+// object as |cert_tree| parameter, otherwise SNI does not work.  All
+// the created SSL_CTX is stored into |all_ssl_ctx|.
+SSL_CTX *setup_server_ssl_context(std::vector<SSL_CTX *> &all_ssl_ctx,
+                                  CertLookupTree *cert_tree
+#ifdef HAVE_NEVERBLEED
+                                  ,
+                                  neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+                                  );
+
+// Setups client side SSL_CTX.  This function inspects get_config()
+// and if downstream_no_tls is true, returns nullptr.  Otherwise, only
+// construct SSL_CTX if either client_mode or http2_bridge is true.
+SSL_CTX *setup_client_ssl_context(
+#ifdef HAVE_NEVERBLEED
+    neverbleed_t *nb
+#endif // HAVE_NEVERBLEED
+    );
+
+// Creates CertLookupTree.  If frontend is configured not to use TLS,
+// this function returns nullptr.
+CertLookupTree *create_cert_lookup_tree();
+
+SSL *create_ssl(SSL_CTX *ssl_ctx);
+
+// Returns true if SSL/TLS is enabled on downstream
+bool downstream_tls_enabled();
+
+// Performs TLS hostname match.  |pattern| of length |plen| can
+// contain wildcard character '*', which matches prefix of target
+// hostname.  There are several restrictions to make wildcard work.
+// The matching algorithm is based on RFC 6125.
+bool tls_hostname_match(const char *pattern, size_t plen, const char *hostname,
+                        size_t hlen);
 
 } // namespace ssl
 
