@@ -25,10 +25,15 @@
 #ifndef SHRPX_SSL_TEST_H
 #define SHRPX_SSL_TEST_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
 namespace shrpx {
 
 void test_shrpx_ssl_create_lookup_tree(void);
 void test_shrpx_ssl_cert_lookup_tree_add_cert_from_file(void);
+void test_shrpx_ssl_tls_hostname_match(void);
 
 } // namespace shrpx
 
