@@ -22,8 +22,15 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
+#ifndef SSL_H
+#define SSL_H
+
 #include "nghttp2_config.h"
 
+#include <cinttypes>
+
+#include <openssl/ssl.h>
+
 namespace nghttp2 {
 
 namespace ssl {
@@ -40,6 +47,31 @@ public:
 
 extern const char *const DEFAULT_CIPHER_LIST;
 
+const char *get_tls_protocol(SSL *ssl);
+
+struct TLSSessionInfo {
+  const char *cipher;
+  const char *protocol;
+  const uint8_t *session_id;
+  bool session_reused;
+  size_t session_id_length;
+};
+
+TLSSessionInfo *get_tls_session_info(TLSSessionInfo *tls_info, SSL *ssl);
+
+// Returns true if SSL/TLS requirement for HTTP/2 is fulfilled.
+// To fulfill the requirement, the following 2 terms must be hold:
+//
+// 1. The negotiated protocol must be TLSv1.2.
+// 2. The negotiated cipher cuite is not listed in the black list
+//    described in RFC 7540.
+bool check_http2_requirement(SSL *ssl);
+
+// Initializes OpenSSL library
+void libssl_init();
+
 } // namespace ssl
 
 } // namespace nghttp2
+
+#endif // SSL_H
