@@ -22,17 +22,22 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
+
 #include <stdio.h>
 #include <string.h>
 #include <CUnit/Basic.h>
 /* include test cases' include files here */
+
 #include "failmalloc_test.h"
 
 static int init_suite1(void) { return 0; }
 
 static int clean_suite1(void) { return 0; }
 
-int main(int argc, char *argv[]) {
+int main(int argc _U_, char *argv[] _U_) {
   CU_pSuite pSuite = NULL;
   unsigned int num_tests_failed;
 
@@ -64,7 +69,7 @@ int main(int argc, char *argv[]) {
   num_tests_failed = CU_get_number_of_tests_failed();
   CU_cleanup_registry();
   if (CU_get_error() == CUE_SUCCESS) {
-    return num_tests_failed;
+    return (int)num_tests_failed;
   } else {
     printf("CUnit Error: %s\n", CU_get_error_msg());
     return CU_get_error();
