@@ -1007,25 +1007,20 @@ void test_nghttp2_hd_deflate_inflate(void) {
   nghttp2_hd_deflater deflater;
   nghttp2_hd_inflater inflater;
   nghttp2_nv nv1[] = {
-      MAKE_NV(":status", "200 OK"),
-      MAKE_NV("access-control-allow-origin", "*"),
+      MAKE_NV(":status", "200 OK"), MAKE_NV("access-control-allow-origin", "*"),
       MAKE_NV("cache-control", "private, max-age=0, must-revalidate"),
-      MAKE_NV("content-length", "76073"),
-      MAKE_NV("content-type", "text/html"),
+      MAKE_NV("content-length", "76073"), MAKE_NV("content-type", "text/html"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
       MAKE_NV("expires", "Sat, 27 Jul 2013 06:22:12 GMT"),
-      MAKE_NV("server", "Apache"),
-      MAKE_NV("vary", "foobar"),
+      MAKE_NV("server", "Apache"), MAKE_NV("vary", "foobar"),
       MAKE_NV("via", "1.1 alphabravo (squid/3.x.x), 1.1 nghttpx"),
       MAKE_NV("x-cache", "MISS from alphabravo"),
-      MAKE_NV("x-cache-action", "MISS"),
-      MAKE_NV("x-cache-age", "0"),
+      MAKE_NV("x-cache-action", "MISS"), MAKE_NV("x-cache-age", "0"),
       MAKE_NV("x-cache-lookup", "MISS from alphabravo:3128"),
       MAKE_NV("x-lb-nocache", "true"),
   };
   nghttp2_nv nv2[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=56682045"),
       MAKE_NV("content-type", "text/css"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1036,8 +1031,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache", "HIT from alphabravo"),
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128")};
   nghttp2_nv nv3[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=56682072"),
       MAKE_NV("content-type", "text/css"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1049,8 +1043,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128"),
   };
   nghttp2_nv nv4[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=56682022"),
       MAKE_NV("content-type", "text/css"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1062,8 +1055,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128"),
   };
   nghttp2_nv nv5[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=4461139"),
       MAKE_NV("content-type", "application/x-javascript"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1075,8 +1067,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128"),
   };
   nghttp2_nv nv6[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=18645951"),
       MAKE_NV("content-type", "application/x-javascript"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1088,8 +1079,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128"),
   };
   nghttp2_nv nv7[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=31536000"),
       MAKE_NV("content-type", "application/javascript"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1101,8 +1091,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128"),
   };
   nghttp2_nv nv8[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=31536000"),
       MAKE_NV("content-type", "application/javascript"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1114,8 +1103,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128"),
   };
   nghttp2_nv nv9[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=31536000"),
       MAKE_NV("content-type", "application/javascript"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
@@ -1127,8 +1115,7 @@ void test_nghttp2_hd_deflate_inflate(void) {
       MAKE_NV("x-cache-lookup", "HIT from alphabravo:3128"),
   };
   nghttp2_nv nv10[] = {
-      MAKE_NV(":status", "304 Not Modified"),
-      MAKE_NV("age", "0"),
+      MAKE_NV(":status", "304 Not Modified"), MAKE_NV("age", "0"),
       MAKE_NV("cache-control", "max-age=56682045"),
       MAKE_NV("content-type", "text/css"),
       MAKE_NV("date", "Sat, 27 Jul 2013 06:22:12 GMT"),
